try:
    import unittest2 as unittest
except ImportError:
    import unittest

from rope.base import exceptions
from rope.contrib import generate
from ropetest import testutils


class GenerateTest(unittest.TestCase):

    def setUp(self):
        super(GenerateTest, self).setUp()
        self.project = testutils.sample_project()
        self.pycore = self.project.pycore
        self.mod = testutils.create_module(self.project, 'mod1')
        self.mod2 = testutils.create_module(self.project, 'mod2')
        self.pkg = testutils.create_package(self.project, 'pkg')

    def tearDown(self):
        testutils.remove_project(self.project)
        super(GenerateTest, self).tearDown()

    def _get_generate(self, offset):
        return generate.GenerateVariable(self.project, self.mod, offset)

    def _get_generate_class(self, offset, goal_mod=None):
        return generate.GenerateClass(self.project, self.mod, offset, goal_resource=goal_mod)

    def _get_generate_module(self, offset):
        return generate.GenerateModule(self.project, self.mod, offset)

    def _get_generate_package(self, offset):
        return generate.GeneratePackage(self.project, self.mod, offset)

    def _get_generate_function(self, offset):
        return generate.GenerateFunction(self.project, self.mod, offset)

    def test_getting_location(self):
        code = 'a_var = name\n'
        self.mod.write(code)
        generator = self._get_generate(code.index('name'))
        self.assertEqual((self.mod, 1), generator.get_location())

    def test_generating_variable(self):
        code = 'a_var = name\n'
        self.mod.write(code)
        changes = self._get_generate(code.index('name')).get_changes()
        self.project.do(changes)
        self.assertEqual('name = None\n\n\na_var = name\n', self.mod.read())

    def test_generating_variable_inserting_before_statement(self):
        code = 'c = 1\nc = b\n'
        self.mod.write(code)
        changes = self._get_generate(code.index('b')).get_changes()
        self.project.do(changes)
        self.assertEqual('c = 1\nb = None\n\n\nc = b\n', self.mod.read())

    def test_generating_variable_in_local_scopes(self):
        code = 'def f():\n    c = 1\n    c = b\n'
        self.mod.write(code)
        changes = self._get_generate(code.index('b')).get_changes()
        self.project.do(changes)
        self.assertEqual('def f():\n    c = 1\n    b = None\n    c = b\n',
                          self.mod.read())

    def test_generating_variable_in_other_modules(self):
        code = 'import mod2\nc = mod2.b\n'
        self.mod.write(code)
        generator = self._get_generate(code.index('b'))
        self.project.do(generator.get_changes())
        self.assertEqual((self.mod2, 1), generator.get_location())
        self.assertEqual('b = None\n', self.mod2.read())

    def test_generating_variable_in_classes(self):
        code = 'class C(object):\n    def f(self):\n        pass\n' \
               'c = C()\na_var = c.attr'
        self.mod.write(code)
        changes = self._get_generate(code.index('attr')).get_changes()
        self.project.do(changes)
        self.assertEqual(
            'class C(object):\n    def f(self):\n        '
            'pass\n\n    attr = None\n'
            'c = C()\na_var = c.attr', self.mod.read())

    def test_generating_variable_in_classes_removing_pass(self):
        code = 'class C(object):\n    pass\nc = C()\na_var = c.attr'
        self.mod.write(code)
        changes = self._get_generate(code.index('attr')).get_changes()
        self.project.do(changes)
        self.assertEqual('class C(object):\n\n    attr = None\n'
                          'c = C()\na_var = c.attr', self.mod.read())

    def test_generating_variable_in_packages(self):
        code = 'import pkg\na = pkg.a\n'
        self.mod.write(code)
        generator = self._get_generate(code.rindex('a'))
        self.project.do(generator.get_changes())
        init = self.pkg.get_child('__init__.py')
        self.assertEqual((init, 1), generator.get_location())
        self.assertEqual('a = None\n', init.read())

    def test_generating_classes(self):
        code = 'c = C()\n'
        self.mod.write(code)
        changes = self._get_generate_class(code.index('C')).get_changes()
        self.project.do(changes)
        self.assertEqual('class C(object):\n    pass\n\n\nc = C()\n',
                          self.mod.read())

    def test_generating_classes_in_other_module(self):
        code = 'c = C()\n'
        self.mod.write(code)
        changes = self._get_generate_class(code.index('C'), self.mod2).get_changes()
        self.project.do(changes)
        self.assertEqual('class C(object):\n    pass\n',
                          self.mod2.read())
        self.assertEqual('from mod2 import C\nc = C()\n',
                          self.mod.read())

    def test_generating_modules(self):
        code = 'import pkg\npkg.mod\n'
        self.mod.write(code)
        generator = self._get_generate_module(code.rindex('mod'))
        self.project.do(generator.get_changes())
        mod = self.pkg.get_child('mod.py')
        self.assertEqual((mod, 1), generator.get_location())
        self.assertEqual('import pkg.mod\npkg.mod\n', self.mod.read())

    def test_generating_packages(self):
        code = 'import pkg\npkg.pkg2\n'
        self.mod.write(code)
        generator = self._get_generate_package(code.rindex('pkg2'))
        self.project.do(generator.get_changes())
        pkg2 = self.pkg.get_child('pkg2')
        init = pkg2.get_child('__init__.py')
        self.assertEqual((init, 1), generator.get_location())
        self.assertEqual('import pkg.pkg2\npkg.pkg2\n', self.mod.read())

    def test_generating_function(self):
        code = 'a_func()\n'
        self.mod.write(code)
        changes = self._get_generate_function(
            code.index('a_func')).get_changes()
        self.project.do(changes)
        self.assertEqual('def a_func():\n    pass\n\n\na_func()\n',
                          self.mod.read())

    def test_generating_modules_with_empty_primary(self):
        code = 'mod\n'
        self.mod.write(code)
        generator = self._get_generate_module(code.rindex('mod'))
        self.project.do(generator.get_changes())
        mod = self.project.root.get_child('mod.py')
        self.assertEqual((mod, 1), generator.get_location())
        self.assertEqual('import mod\nmod\n', self.mod.read())

    def test_generating_variable_already_exists(self):
        code = 'b = 1\nc = b\n'
        self.mod.write(code)
        with self.assertRaises(exceptions.RefactoringError):
            self._get_generate(code.index('b')).get_changes()

    def test_generating_variable_primary_cannot_be_determined(self):
        code = 'c = can_not_be_found.b\n'
        self.mod.write(code)
        with self.assertRaises(exceptions.RefactoringError):
            self._get_generate(code.rindex('b')).get_changes()

    def test_generating_modules_when_already_exists(self):
        code = 'mod2\n'
        self.mod.write(code)
        generator = self._get_generate_module(code.rindex('mod'))
        with self.assertRaises(exceptions.RefactoringError):
            self.project.do(generator.get_changes())

    def test_generating_static_methods(self):
        code = 'class C(object):\n    pass\nC.a_func()\n'
        self.mod.write(code)
        changes = self._get_generate_function(
            code.index('a_func')).get_changes()
        self.project.do(changes)
        self.assertEqual(
            'class C(object):\n\n    @staticmethod'
            '\n    def a_func():\n        pass\nC.a_func()\n',
            self.mod.read())

    def test_generating_methods(self):
        code = 'class C(object):\n    pass\nc = C()\nc.a_func()\n'
        self.mod.write(code)
        changes = self._get_generate_function(
            code.index('a_func')).get_changes()
        self.project.do(changes)
        self.assertEqual(
            'class C(object):\n\n    def a_func(self):\n        pass\n'
            'c = C()\nc.a_func()\n',
            self.mod.read())

    def test_generating_constructors(self):
        code = 'class C(object):\n    pass\nc = C()\n'
        self.mod.write(code)
        changes = self._get_generate_function(code.rindex('C')).get_changes()
        self.project.do(changes)
        self.assertEqual(
            'class C(object):\n\n    def __init__(self):\n        pass\n'
            'c = C()\n',
            self.mod.read())

    def test_generating_calls(self):
        code = 'class C(object):\n    pass\nc = C()\nc()\n'
        self.mod.write(code)
        changes = self._get_generate_function(code.rindex('c')).get_changes()
        self.project.do(changes)
        self.assertEqual(
            'class C(object):\n\n    def __call__(self):\n        pass\n'
            'c = C()\nc()\n',
            self.mod.read())

    def test_generating_calls_in_other_modules(self):
        self.mod2.write('class C(object):\n    pass\n')
        code = 'import mod2\nc = mod2.C()\nc()\n'
        self.mod.write(code)
        changes = self._get_generate_function(code.rindex('c')).get_changes()
        self.project.do(changes)
        self.assertEqual(
            'class C(object):\n\n    def __call__(self):\n        pass\n',
            self.mod2.read())

    def test_generating_function_handling_arguments(self):
        code = 'a_func(1)\n'
        self.mod.write(code)
        changes = self._get_generate_function(
            code.index('a_func')).get_changes()
        self.project.do(changes)
        self.assertEqual('def a_func(arg0):\n    pass\n\n\na_func(1)\n',
                          self.mod.read())

    def test_generating_function_handling_keyword_xarguments(self):
        code = 'a_func(p=1)\n'
        self.mod.write(code)
        changes = self._get_generate_function(
            code.index('a_func')).get_changes()
        self.project.do(changes)
        self.assertEqual('def a_func(p):\n    pass\n\n\na_func(p=1)\n',
                          self.mod.read())

    def test_generating_function_handling_arguments_better_naming(self):
        code = 'a_var = 1\na_func(a_var)\n'
        self.mod.write(code)
        changes = self._get_generate_function(
            code.index('a_func')).get_changes()
        self.project.do(changes)
        self.assertEqual('a_var = 1\ndef a_func(a_var):'
                          '\n    pass\n\n\na_func(a_var)\n',
                          self.mod.read())

    def test_generating_variable_in_other_modules2(self):
        self.mod2.write('\n\n\nprint(1)\n')
        code = 'import mod2\nc = mod2.b\n'
        self.mod.write(code)
        generator = self._get_generate(code.index('b'))
        self.project.do(generator.get_changes())
        self.assertEqual((self.mod2, 5), generator.get_location())
        self.assertEqual('\n\n\nprint(1)\n\n\nb = None\n', self.mod2.read())

    def test_generating_function_in_a_suite(self):
        code = 'if True:\n    a_func()\n'
        self.mod.write(code)
        changes = self._get_generate_function(
            code.index('a_func')).get_changes()
        self.project.do(changes)
        self.assertEqual('def a_func():\n    pass'
                          '\n\n\nif True:\n    a_func()\n',
                          self.mod.read())

    def test_generating_function_in_a_suite_in_a_function(self):
        code = 'def f():\n    a = 1\n    if 1:\n        g()\n'
        self.mod.write(code)
        changes = self._get_generate_function(code.index('g()')).get_changes()
        self.project.do(changes)
        self.assertEqual(
            'def f():\n    a = 1\n    def g():\n        pass\n'
            '    if 1:\n        g()\n',
            self.mod.read())

    def test_create_generate_class_with_goal_resource(self):
        code = 'c = C()\n'
        self.mod.write(code)

        result = generate.create_generate(
            "class",
            self.project,
            self.mod,
            code.index("C"),
            goal_resource=self.mod2)

        self.assertTrue(isinstance(result, generate.GenerateClass))
        self.assertEqual(result.goal_resource, self.mod2)

    def test_create_generate_class_without_goal_resource(self):
        code = 'c = C()\n'
        self.mod.write(code)

        result = generate.create_generate(
            "class",
            self.project,
            self.mod,
            code.index("C"))

        self.assertTrue(isinstance(result, generate.GenerateClass))
        self.assertIsNone(result.goal_resource)

if __name__ == '__main__':
    unittest.main()



try:
    import unittest2 as unittest
except ImportError:
    import unittest

from ropetest import testutils
from rope.contrib import autoimport


class AutoImportTest(unittest.TestCase):

    def setUp(self):
        super(AutoImportTest, self).setUp()
        self.project = testutils.sample_project(extension_modules=['sys'])
        self.mod1 = testutils.create_module(self.project, 'mod1')
        self.pkg = testutils.create_package(self.project, 'pkg')
        self.mod2 = testutils.create_module(self.project, 'mod2', self.pkg)
        self.importer = autoimport.AutoImport(self.project, observe=False)

    def tearDown(self):
        testutils.remove_project(self.project)
        super(AutoImportTest, self).tearDown()

    def test_simple_case(self):
        self.assertEqual([], self.importer.import_assist('A'))

    def test_update_resource(self):
        self.mod1.write('myvar = None\n')
        self.importer.update_resource(self.mod1)
        self.assertEqual([('myvar', 'mod1')],
                          self.importer.import_assist('myva'))

    def test_update_module(self):
        self.mod1.write('myvar = None')
        self.importer.update_module('mod1')
        self.assertEqual([('myvar', 'mod1')],
                          self.importer.import_assist('myva'))

    def test_update_non_existent_module(self):
        self.importer.update_module('does_not_exists_this')
        self.assertEqual([], self.importer.import_assist('myva'))

    def test_module_with_syntax_errors(self):
        self.mod1.write('this is a syntax error\n')
        self.importer.update_resource(self.mod1)
        self.assertEqual([], self.importer.import_assist('myva'))

    def test_excluding_imported_names(self):
        self.mod1.write('import pkg\n')
        self.importer.update_resource(self.mod1)
        self.assertEqual([], self.importer.import_assist('pkg'))

    def test_get_modules(self):
        self.mod1.write('myvar = None\n')
        self.importer.update_resource(self.mod1)
        self.assertEqual(['mod1'], self.importer.get_modules('myvar'))

    def test_get_modules_inside_packages(self):
        self.mod1.write('myvar = None\n')
        self.mod2.write('myvar = None\n')
        self.importer.update_resource(self.mod1)
        self.importer.update_resource(self.mod2)
        self.assertEqual(set(['mod1', 'pkg.mod2']),
                          set(self.importer.get_modules('myvar')))

    def test_trivial_insertion_line(self):
        result = self.importer.find_insertion_line('')
        self.assertEqual(1, result)

    def test_insertion_line(self):
        result = self.importer.find_insertion_line('import mod\n')
        self.assertEqual(2, result)

    def test_insertion_line_with_pydocs(self):
        result = self.importer.find_insertion_line(
            '"""docs\n\ndocs"""\nimport mod\n')
        self.assertEqual(5, result)

    def test_insertion_line_with_multiple_imports(self):
        result = self.importer.find_insertion_line(
            'import mod1\n\nimport mod2\n')
        self.assertEqual(4, result)

    def test_insertion_line_with_blank_lines(self):
        result = self.importer.find_insertion_line(
            'import mod1\n\n# comment\n')
        self.assertEqual(2, result)

    def test_empty_cache(self):
        self.mod1.write('myvar = None\n')
        self.importer.update_resource(self.mod1)
        self.assertEqual(['mod1'], self.importer.get_modules('myvar'))
        self.importer.clear_cache()
        self.assertEqual([], self.importer.get_modules('myvar'))

    def test_not_caching_underlined_names(self):
        self.mod1.write('_myvar = None\n')
        self.importer.update_resource(self.mod1, underlined=False)
        self.assertEqual([], self.importer.get_modules('_myvar'))
        self.importer.update_resource(self.mod1, underlined=True)
        self.assertEqual(['mod1'], self.importer.get_modules('_myvar'))

    def test_caching_underlined_names_passing_to_the_constructor(self):
        importer = autoimport.AutoImport(self.project, False, True)
        self.mod1.write('_myvar = None\n')
        importer.update_resource(self.mod1)
        self.assertEqual(['mod1'], importer.get_modules('_myvar'))

    def test_name_locations(self):
        self.mod1.write('myvar = None\n')
        self.importer.update_resource(self.mod1)
        self.assertEqual([(self.mod1, 1)],
                          self.importer.get_name_locations('myvar'))

    def test_name_locations_with_multiple_occurrences(self):
        self.mod1.write('myvar = None\n')
        self.mod2.write('\nmyvar = None\n')
        self.importer.update_resource(self.mod1)
        self.importer.update_resource(self.mod2)
        self.assertEqual(set([(self.mod1, 1), (self.mod2, 2)]),
                          set(self.importer.get_name_locations('myvar')))

    def test_handling_builtin_modules(self):
        self.importer.update_module('sys')
        self.assertTrue('sys' in self.importer.get_modules('exit'))

    def test_submodules(self):
        self.assertEqual(set([self.mod1]),
                          autoimport.submodules(self.mod1))
        self.assertEqual(set([self.mod2, self.pkg]),
                          autoimport.submodules(self.pkg))


class AutoImportObservingTest(unittest.TestCase):

    def setUp(self):
        super(AutoImportObservingTest, self).setUp()
        self.project = testutils.sample_project()
        self.mod1 = testutils.create_module(self.project, 'mod1')
        self.pkg = testutils.create_package(self.project, 'pkg')
        self.mod2 = testutils.create_module(self.project, 'mod2', self.pkg)
        self.importer = autoimport.AutoImport(self.project, observe=True)

    def tearDown(self):
        testutils.remove_project(self.project)
        super(AutoImportObservingTest, self).tearDown()

    def test_writing_files(self):
        self.mod1.write('myvar = None\n')
        self.assertEqual(['mod1'], self.importer.get_modules('myvar'))

    def test_moving_files(self):
        self.mod1.write('myvar = None\n')
        self.mod1.move('mod3.py')
        self.assertEqual(['mod3'], self.importer.get_modules('myvar'))

    def test_removing_files(self):
        self.mod1.write('myvar = None\n')
        self.mod1.remove()
        self.assertEqual([], self.importer.get_modules('myvar'))


def suite():
    result = unittest.TestSuite()
    result.addTests(unittest.makeSuite(AutoImportTest))
    result.addTests(unittest.makeSuite(AutoImportObservingTest))
    return result

if __name__ == '__main__':
    unittest.main()



try:
    import unittest2 as unittest
except ImportError:
    import unittest

from rope.base import libutils
from rope.base.pyobjects import get_base_type
from ropetest import testutils


class PyCoreScopesTest(unittest.TestCase):

    def setUp(self):
        super(PyCoreScopesTest, self).setUp()
        self.project = testutils.sample_project()
        self.pycore = self.project.pycore

    def tearDown(self):
        testutils.remove_project(self.project)
        super(PyCoreScopesTest, self).tearDown()

    def test_simple_scope(self):
        scope = libutils.get_string_scope(
            self.project, 'def sample_func():\n    pass\n')
        sample_func = scope['sample_func'].get_object()
        self.assertEqual(get_base_type('Function'), sample_func.get_type())

    def test_simple_function_scope(self):
        scope = libutils.get_string_scope(
            self.project, 'def sample_func():\n    a = 10\n')
        self.assertEqual(1, len(scope.get_scopes()))
        sample_func_scope = scope.get_scopes()[0]
        self.assertEqual(1, len(sample_func_scope.get_names()))
        self.assertEqual(0, len(sample_func_scope.get_scopes()))

    def test_classes_inside_function_scopes(self):
        scope = libutils.get_string_scope(
            self.project,
            'def sample_func():\n'
            '    class SampleClass(object):\n        pass\n')
        self.assertEqual(1, len(scope.get_scopes()))
        sample_func_scope = scope.get_scopes()[0]  # noqa
        self.assertEqual(get_base_type('Type'),
                          scope.get_scopes()[0]['SampleClass'].
                          get_object().get_type())

    def test_list_comprehension_scope_inside_assignment(self):
        scope = libutils.get_string_scope(
            self.project, 'a_var = [b_var + d_var for b_var, c_var in e_var]\n')
        self.assertEqual(
            list(sorted(scope.get_defined_names())),
            ['a_var', 'b_var', 'c_var'],
        )

    def test_list_comprehension_scope(self):
        scope = libutils.get_string_scope(
            self.project, '[b_var + d_var for b_var, c_var in e_var]\n')
        self.assertEqual(
            list(sorted(scope.get_defined_names())),
            ['b_var', 'c_var'],
        )

    def test_set_comprehension_scope(self):
        scope = libutils.get_string_scope(
            self.project, '{b_var + d_var for b_var, c_var in e_var}\n')
        self.assertEqual(
            list(sorted(scope.get_defined_names())),
            ['b_var', 'c_var'],
        )

    def test_generator_comprehension_scope(self):
        scope = libutils.get_string_scope(
            self.project, '(b_var + d_var for b_var, c_var in e_var)\n')
        self.assertEqual(
            list(sorted(scope.get_defined_names())),
            ['b_var', 'c_var'],
        )

    def test_dict_comprehension_scope(self):
        scope = libutils.get_string_scope(
            self.project, '{b_var: d_var for b_var, c_var in e_var}\n')
        self.assertEqual(
            list(sorted(scope.get_defined_names())),
            ['b_var', 'c_var'],
        )

    @testutils.only_for_versions_higher('3.8')
    def test_inline_assignment_in_comprehensions(self):
        scope = libutils.get_string_scope(
            self.project, '''[
                (a_var := b_var + (f_var := g_var))
                for b_var in [(j_var := i_var)
                for i_var in c_var] if a_var + (h_var := d_var)
            ]''')
        self.assertEqual(
            list(sorted(scope.get_defined_names())),
            ['a_var', 'b_var', 'f_var', 'h_var', 'i_var', 'j_var'],
        )

    def test_nested_comprehension(self):
        scope = libutils.get_string_scope(
            self.project, '''[
                b_var + d_var for b_var, c_var in [
                    e_var for e_var in f_var
                ]
            ]\n''')
        self.assertEqual(
            list(sorted(scope.get_defined_names())),
            ['b_var', 'c_var', 'e_var'],
        )

    def test_simple_class_scope(self):
        scope = libutils.get_string_scope(
            self.project,
            'class SampleClass(object):\n'
            '    def f(self):\n        var = 10\n')
        self.assertEqual(1, len(scope.get_scopes()))
        sample_class_scope = scope.get_scopes()[0]
        self.assertTrue('f' in sample_class_scope)
        self.assertEqual(1, len(sample_class_scope.get_scopes()))
        f_in_class = sample_class_scope.get_scopes()[0]
        self.assertTrue('var' in f_in_class)

    def test_get_lineno(self):
        scope = libutils.get_string_scope(
            self.project, '\ndef sample_func():\n    a = 10\n')
        self.assertEqual(1, len(scope.get_scopes()))
        sample_func_scope = scope.get_scopes()[0]
        self.assertEqual(1, scope.get_start())
        self.assertEqual(2, sample_func_scope.get_start())

    def test_scope_kind(self):
        scope = libutils.get_string_scope(
            self.project,
            'class SampleClass(object):\n    pass\n'
            'def sample_func():\n    pass\n')
        sample_class_scope = scope.get_scopes()[0]
        sample_func_scope = scope.get_scopes()[1]
        self.assertEqual('Module', scope.get_kind())
        self.assertEqual('Class', sample_class_scope.get_kind())
        self.assertEqual('Function', sample_func_scope.get_kind())

    def test_function_parameters_in_scope_names(self):
        scope = libutils.get_string_scope(
            self.project, 'def sample_func(param):\n    a = 10\n')
        sample_func_scope = scope.get_scopes()[0]
        self.assertTrue('param' in sample_func_scope)

    def test_get_names_contains_only_names_defined_in_a_scope(self):
        scope = libutils.get_string_scope(
            self.project,
            'var1 = 10\ndef sample_func(param):\n    var2 = 20\n')
        sample_func_scope = scope.get_scopes()[0]
        self.assertTrue('var1' not in sample_func_scope)

    def test_scope_lookup(self):
        scope = libutils.get_string_scope(
            self.project,
            'var1 = 10\ndef sample_func(param):\n    var2 = 20\n')
        self.assertTrue(scope.lookup('var2') is None)
        self.assertEqual(get_base_type('Function'),
                          scope.lookup('sample_func').get_object().get_type())
        sample_func_scope = scope.get_scopes()[0]
        self.assertTrue(sample_func_scope.lookup('var1') is not None)

    def test_function_scopes(self):
        scope = libutils.get_string_scope(
            self.project, 'def func():\n    var = 10\n')
        func_scope = scope.get_scopes()[0]
        self.assertTrue('var' in func_scope)

    def test_function_scopes_classes(self):
        scope = libutils.get_string_scope(
            self.project,
            'def func():\n    class Sample(object):\n        pass\n')
        func_scope = scope.get_scopes()[0]
        self.assertTrue('Sample' in func_scope)

    def test_function_getting_scope(self):
        mod = libutils.get_string_module(
            self.project, 'def func():    var = 10\n')
        func_scope = mod['func'].get_object().get_scope()
        self.assertTrue('var' in func_scope)

    def test_scopes_in_function_scopes(self):
        scope = libutils.get_string_scope(
            self.project,
            'def func():\n    def inner():\n        var = 10\n')
        func_scope = scope.get_scopes()[0]
        inner_scope = func_scope.get_scopes()[0]
        self.assertTrue('var' in inner_scope)

    def test_for_variables_in_scopes(self):
        scope = libutils.get_string_scope(
            self.project, 'for a_var in range(10):\n    pass\n')
        self.assertTrue('a_var' in scope)

    def test_assists_inside_fors(self):
        scope = libutils.get_string_scope(
            self.project, 'for i in range(10):\n    a_var = i\n')
        self.assertTrue('a_var' in scope)

    def test_first_parameter_of_a_method(self):
        code = 'class AClass(object):\n' \
               '    def a_func(self, param):\n        pass\n'
        a_class = libutils.get_string_module(self.project, code)['AClass'].\
            get_object()
        function_scope = a_class['a_func'].get_object().get_scope()
        self.assertEqual(a_class,
                          function_scope['self'].get_object().get_type())
        self.assertNotEqual(a_class, function_scope['param'].
                             get_object().get_type())

    def test_first_parameter_of_static_methods(self):
        code = 'class AClass(object):\n' \
               '    @staticmethod\n    def a_func(param):\n        pass\n'
        a_class = libutils.get_string_module(self.project, code)['AClass'].\
            get_object()
        function_scope = a_class['a_func'].\
            get_object().get_scope()
        self.assertNotEqual(a_class,
                             function_scope['param'].get_object().get_type())

    def test_first_parameter_of_class_methods(self):
        code = 'class AClass(object):\n' \
            '    @classmethod\n    def a_func(cls):\n        pass\n'
        a_class = libutils.get_string_module(self.project, code)['AClass'].\
            get_object()
        function_scope = a_class['a_func'].get_object().get_scope()
        self.assertEqual(a_class, function_scope['cls'].get_object())

    def test_first_parameter_with_self_as_name_and_unknown_decorator(self):
        code = 'def my_decorator(func):\n    return func\n'\
               'class AClass(object):\n' \
               '    @my_decorator\n    def a_func(self):\n        pass\n'
        a_class = libutils.get_string_module(self.project, code)['AClass'].\
            get_object()
        function_scope = a_class['a_func'].get_object().get_scope()
        self.assertEqual(a_class, function_scope['self'].
                          get_object().get_type())

    def test_inside_class_scope_attribute_lookup(self):
        scope = libutils.get_string_scope(
            self.project,
            'class C(object):\n'
            '    an_attr = 1\n'
            '    def a_func(self):\n        pass')
        self.assertEqual(1, len(scope.get_scopes()))
        c_scope = scope.get_scopes()[0]
        self.assertTrue('an_attr'in c_scope.get_names())
        self.assertTrue(c_scope.lookup('an_attr') is not None)
        f_in_c = c_scope.get_scopes()[0]
        self.assertTrue(f_in_c.lookup('an_attr') is None)

    def test_inside_class_scope_attribute_lookup2(self):
        scope = libutils.get_string_scope(
            self.project,
            'class C(object):\n'
            '    def __init__(self):\n        self.an_attr = 1\n'
            '    def a_func(self):\n        pass')
        self.assertEqual(1, len(scope.get_scopes()))
        c_scope = scope.get_scopes()[0]
        f_in_c = c_scope.get_scopes()[0]
        self.assertTrue(f_in_c.lookup('an_attr') is None)

    def test_get_inner_scope_for_staticmethods(self):
        scope = libutils.get_string_scope(
            self.project,
            'class C(object):\n'
            '    @staticmethod\n'
            '    def a_func(self):\n        pass\n')
        c_scope = scope.get_scopes()[0]
        f_in_c = c_scope.get_scopes()[0]
        self.assertEqual(f_in_c, scope.get_inner_scope_for_line(4))

    def test_getting_overwritten_scopes(self):
        scope = libutils.get_string_scope(
            self.project, 'def f():\n    pass\ndef f():\n    pass\n')
        self.assertEqual(2, len(scope.get_scopes()))
        f1_scope = scope.get_scopes()[0]
        f2_scope = scope.get_scopes()[1]
        self.assertNotEqual(f1_scope, f2_scope)

    def test_assigning_builtin_names(self):
        mod = libutils.get_string_module(self.project, 'range = 1\n')
        range = mod.get_scope().lookup('range')
        self.assertEqual((mod, 1), range.get_definition_location())

    def test_get_inner_scope_and_logical_lines(self):
        scope = libutils.get_string_scope(
            self.project,
            'class C(object):\n'
            '    def f():\n        s = """\n1\n2\n"""\n        a = 1\n')
        c_scope = scope.get_scopes()[0]
        f_in_c = c_scope.get_scopes()[0]
        self.assertEqual(f_in_c, scope.get_inner_scope_for_line(7))

    def test_getting_defined_names_for_classes(self):
        scope = libutils.get_string_scope(
            self.project,
            'class A(object):\n    def a(self):\n        pass\n'
            'class B(A):\n    def b(self):\n        pass\n')
        a_scope = scope['A'].get_object().get_scope()  # noqa
        b_scope = scope['B'].get_object().get_scope()
        self.assertTrue('a' in b_scope.get_names())
        self.assertTrue('b' in b_scope.get_names())
        self.assertTrue('a' not in b_scope.get_defined_names())
        self.assertTrue('b' in b_scope.get_defined_names())

    def test_getting_defined_names_for_modules(self):
        scope = libutils.get_string_scope(
            self.project, 'class A(object):\n    pass\n')
        self.assertTrue('open' in scope.get_names())
        self.assertTrue('A' in scope.get_names())
        self.assertTrue('open' not in scope.get_defined_names())
        self.assertTrue('A' in scope.get_defined_names())


def suite():
    result = unittest.TestSuite()
    result.addTests(unittest.makeSuite(PyCoreScopesTest))
    return result


if __name__ == '__main__':
    unittest.main()



import sys
from textwrap import dedent
try:
    import unittest2 as unittest
except ImportError:
    import unittest

import rope.base.codeanalyze
import rope.refactor.occurrences
from rope.refactor import rename
from rope.refactor.rename import Rename
from ropetest import testutils


class RenameRefactoringTest(unittest.TestCase):

    def setUp(self):
        super(RenameRefactoringTest, self).setUp()
        self.project = testutils.sample_project()

    def tearDown(self):
        testutils.remove_project(self.project)
        super(RenameRefactoringTest, self).tearDown()

    def _local_rename(self, source_code, offset, new_name):
        testmod = testutils.create_module(self.project, 'testmod')
        testmod.write(source_code)
        changes = Rename(self.project, testmod, offset).\
            get_changes(new_name, resources=[testmod])
        self.project.do(changes)
        return testmod.read()

    def _rename(self, resource, offset, new_name, **kwds):
        changes = Rename(self.project, resource, offset).\
            get_changes(new_name, **kwds)
        self.project.do(changes)

    def test_simple_global_variable_renaming(self):
        refactored = self._local_rename('a_var = 20\n', 2, 'new_var')
        self.assertEqual('new_var = 20\n', refactored)

    def test_variable_renaming_only_in_its_scope(self):
        refactored = self._local_rename(
            'a_var = 20\ndef a_func():\n    a_var = 10\n', 32, 'new_var')
        self.assertEqual('a_var = 20\ndef a_func():\n    new_var = 10\n',
                          refactored)

    def test_not_renaming_dot_name(self):
        refactored = self._local_rename(
            "replace = True\n'aaa'.replace('a', 'b')\n", 1, 'new_var')
        self.assertEqual("new_var = True\n'aaa'.replace('a', 'b')\n",
                          refactored)

    def test_renaming_multiple_names_in_the_same_line(self):
        refactored = self._local_rename(
            'a_var = 10\na_var = 10 + a_var / 2\n', 2, 'new_var')
        self.assertEqual('new_var = 10\nnew_var = 10 + new_var / 2\n',
                          refactored)

    def test_renaming_names_when_getting_some_attribute(self):
        refactored = self._local_rename(
            "a_var = 'a b c'\na_var.split('\\n')\n", 2, 'new_var')
        self.assertEqual("new_var = 'a b c'\nnew_var.split('\\n')\n",
                          refactored)

    def test_renaming_names_when_getting_some_attribute2(self):
        refactored = self._local_rename(
            "a_var = 'a b c'\na_var.split('\\n')\n", 20, 'new_var')
        self.assertEqual("new_var = 'a b c'\nnew_var.split('\\n')\n",
                          refactored)

    def test_renaming_function_parameters1(self):
        refactored = self._local_rename(
            "def f(a_param):\n    print(a_param)\n", 8, 'new_param')
        self.assertEqual("def f(new_param):\n    print(new_param)\n",
                          refactored)

    def test_renaming_function_parameters2(self):
        refactored = self._local_rename(
            "def f(a_param):\n    print(a_param)\n", 30, 'new_param')
        self.assertEqual("def f(new_param):\n    print(new_param)\n",
                          refactored)

    def test_renaming_occurrences_inside_functions(self):
        code = 'def a_func(p1):\n    a = p1\na_func(1)\n'
        refactored = self._local_rename(code, code.index('p1') + 1,
                                        'new_param')
        self.assertEqual(
            'def a_func(new_param):\n    a = new_param\na_func(1)\n',
            refactored)

    def test_renaming_comprehension_loop_variables(self):
        code = '[b_var for b_var, c_var in d_var if b_var == c_var]'
        refactored = self._local_rename(code, code.index('b_var') + 1,
                                        'new_var')
        self.assertEqual(
            '[new_var for new_var, c_var in d_var if new_var == c_var]',
            refactored)

    def test_renaming_list_comprehension_loop_variables_in_assignment(self):
        code = 'a_var = [b_var for b_var, c_var in d_var if b_var == c_var]'
        refactored = self._local_rename(code, code.index('b_var') + 1,
                                        'new_var')
        self.assertEqual(
            'a_var = [new_var for new_var, c_var in d_var if new_var == c_var]',
            refactored)

    def test_renaming_generator_comprehension_loop_variables(self):
        code = 'a_var = (b_var for b_var, c_var in d_var if b_var == c_var)'
        refactored = self._local_rename(code, code.index('b_var') + 1,
                                        'new_var')
        self.assertEqual(
            'a_var = (new_var for new_var, c_var in d_var if new_var == c_var)',
            refactored)

    @unittest.expectedFailure
    def test_renaming_comprehension_loop_variables_scope(self):
        # FIXME: variable scoping for comprehensions is incorrect, we currently
        #        don't create a scope for comprehension
        code = dedent('''\
            [b_var for b_var, c_var in d_var if b_var == c_var]
            b_var = 10
        ''')
        refactored = self._local_rename(code, code.index('b_var') + 1,
                                        'new_var')
        self.assertEqual(
            '[new_var for new_var, c_var in d_var if new_var == c_var]\nb_var = 10\n',
            refactored)

    @testutils.only_for_versions_higher('3.8')
    def test_renaming_inline_assignment(self):
        code = dedent('''\
            while a_var := next(foo):
                print(a_var)
        ''')
        refactored = self._local_rename(code, code.index('a_var') + 1,
                                        'new_var')
        self.assertEqual(
            dedent('''\
                while new_var := next(foo):
                    print(new_var)
            '''),
            refactored,
        )

    def test_renaming_arguments_for_normal_args_changing_calls(self):
        code = 'def a_func(p1=None, p2=None):\n    pass\na_func(p2=1)\n'
        refactored = self._local_rename(code, code.index('p2') + 1, 'p3')
        self.assertEqual(
            'def a_func(p1=None, p3=None):\n    pass\na_func(p3=1)\n',
            refactored)

    def test_renaming_function_parameters_of_class_init(self):
        code = 'class A(object):\n    def __init__(self, a_param):' \
               '\n        pass\n' \
               'a_var = A(a_param=1)\n'
        refactored = self._local_rename(code, code.index('a_param') + 1,
                                        'new_param')
        expected = 'class A(object):\n    ' \
            'def __init__(self, new_param):\n        pass\n' \
            'a_var = A(new_param=1)\n'
        self.assertEqual(expected, refactored)

    def test_rename_functions_parameters_and_occurences_in_other_modules(self):
        mod1 = testutils.create_module(self.project, 'mod1')
        mod2 = testutils.create_module(self.project, 'mod2')
        mod1.write('def a_func(a_param):\n    print(a_param)\n')
        mod2.write('from mod1 import a_func\na_func(a_param=10)\n')
        self._rename(mod1, mod1.read().index('a_param') + 1, 'new_param')
        self.assertEqual('def a_func(new_param):\n    print(new_param)\n',
                          mod1.read())
        self.assertEqual('from mod1 import a_func\na_func(new_param=10)\n',
                          mod2.read())

    def test_renaming_with_backslash_continued_names(self):
        refactored = self._local_rename(
            "replace = True\n'ali'.\\\nreplace\n", 2, 'is_replace')
        self.assertEqual("is_replace = True\n'ali'.\\\nreplace\n",
                          refactored)

    @testutils.only_for('3.6')
    def test_renaming_occurrence_in_f_string(self):
        refactored = self._local_rename(
            "a_var = 20\na_string=f'value: {a_var}'\n", 2, 'new_var')
        self.assertEqual("new_var = 20\na_string=f'value: {new_var}'\n",
                          refactored)

    @testutils.only_for('3.6')
    def test_renaming_occurrence_in_nested_f_string(self):
        refactored = self._local_rename(
            "a_var = 20\na_string=f'{f\"{a_var}\"}'\n", 2, 'new_var')
        self.assertEqual(
            "new_var = 20\na_string=f'{f\"{new_var}\"}'\n",
            refactored)

    @testutils.only_for('3.6')
    def test_not_renaming_string_contents_in_f_string(self):
        refactored = self._local_rename(
            "a_var = 20\na_string=f'{\"a_var\"}'\n", 2, 'new_var')
        self.assertEqual("new_var = 20\na_string=f'{\"a_var\"}'\n",
                          refactored)

    def test_not_renaming_string_contents(self):
        refactored = self._local_rename("a_var = 20\na_string='a_var'\n",
                                        2, 'new_var')
        self.assertEqual("new_var = 20\na_string='a_var'\n",
                          refactored)

    def test_not_renaming_comment_contents(self):
        refactored = self._local_rename("a_var = 20\n# a_var\n",
                                        2, 'new_var')
        self.assertEqual("new_var = 20\n# a_var\n", refactored)

    def test_renaming_all_occurances_in_containing_scope(self):
        code = 'if True:\n    a_var = 1\nelse:\n    a_var = 20\n'
        refactored = self._local_rename(code, 16, 'new_var')
        self.assertEqual(
            'if True:\n    new_var = 1\nelse:\n    new_var = 20\n', refactored)

    def test_renaming_a_variable_with_arguement_name(self):
        code = 'a_var = 10\ndef a_func(a_var):\n    print(a_var)\n'
        refactored = self._local_rename(code, 1, 'new_var')
        self.assertEqual(
            'new_var = 10\ndef a_func(a_var):\n    print(a_var)\n', refactored)

    def test_renaming_an_arguement_with_variable_name(self):
        code = 'a_var = 10\ndef a_func(a_var):\n    print(a_var)\n'
        refactored = self._local_rename(code, len(code) - 3, 'new_var')
        self.assertEqual(
            'a_var = 10\ndef a_func(new_var):\n    print(new_var)\n',
            refactored)

    def test_renaming_function_with_local_variable_name(self):
        code = 'def a_func():\n    a_func=20\na_func()'
        refactored = self._local_rename(code, len(code) - 3, 'new_func')
        self.assertEqual('def new_func():\n    a_func=20\nnew_func()',
                          refactored)

    def test_renaming_functions(self):
        code = 'def a_func():\n    pass\na_func()\n'
        refactored = self._local_rename(code, len(code) - 5, 'new_func')
        self.assertEqual('def new_func():\n    pass\nnew_func()\n',
                          refactored)

    @testutils.only_for('3.5')
    def test_renaming_async_function(self):
        code = 'async def a_func():\n    pass\na_func()'
        refactored = self._local_rename(code, len(code) - 5, 'new_func')
        self.assertEqual('async def new_func():\n    pass\nnew_func()',
                          refactored)

    @testutils.only_for('3.5')
    def test_renaming_await(self):
        code = 'async def b_func():\n    pass\nasync def a_func():\n    await b_func()'
        refactored = self._local_rename(code, len(code) - 5, 'new_func')
        self.assertEqual('async def new_func():\n    pass\nasync def a_func():\n    await new_func()',
                          refactored)


    def test_renaming_functions_across_modules(self):
        mod1 = testutils.create_module(self.project, 'mod1')
        mod1.write('def a_func():\n    pass\na_func()\n')
        mod2 = testutils.create_module(self.project, 'mod2')
        mod2.write('import mod1\nmod1.a_func()\n')
        self._rename(mod1, len(mod1.read()) - 5, 'new_func')
        self.assertEqual('def new_func():\n    pass\nnew_func()\n',
                          mod1.read())
        self.assertEqual('import mod1\nmod1.new_func()\n', mod2.read())

    def test_renaming_functions_across_modules_from_import(self):
        mod1 = testutils.create_module(self.project, 'mod1')
        mod1.write('def a_func():\n    pass\na_func()\n')
        mod2 = testutils.create_module(self.project, 'mod2')
        mod2.write('from mod1 import a_func\na_func()\n')
        self._rename(mod1, len(mod1.read()) - 5, 'new_func')
        self.assertEqual('def new_func():\n    pass\nnew_func()\n',
                          mod1.read())
        self.assertEqual('from mod1 import new_func\nnew_func()\n',
                          mod2.read())

    def test_renaming_functions_from_another_module(self):
        mod1 = testutils.create_module(self.project, 'mod1')
        mod1.write('def a_func():\n    pass\na_func()\n')
        mod2 = testutils.create_module(self.project, 'mod2')
        mod2.write('import mod1\nmod1.a_func()\n')
        self._rename(mod2, len(mod2.read()) - 5, 'new_func')
        self.assertEqual('def new_func():\n    pass\nnew_func()\n',
                          mod1.read())
        self.assertEqual('import mod1\nmod1.new_func()\n', mod2.read())

    def test_applying_all_changes_together(self):
        mod1 = testutils.create_module(self.project, 'mod1')
        mod1.write('import mod2\nmod2.a_func()\n')
        mod2 = testutils.create_module(self.project, 'mod2')
        mod2.write('def a_func():\n    pass\na_func()\n')
        self._rename(mod2, len(mod2.read()) - 5, 'new_func')
        self.assertEqual('import mod2\nmod2.new_func()\n', mod1.read())
        self.assertEqual('def new_func():\n    pass\nnew_func()\n',
                          mod2.read())

    def test_renaming_modules(self):
        mod1 = testutils.create_module(self.project, 'mod1')
        mod1.write('def a_func():\n    pass\n')
        mod2 = testutils.create_module(self.project, 'mod2')
        mod2.write('from mod1 import a_func\n')
        self._rename(mod2, mod2.read().index('mod1') + 1, 'newmod')
        self.assertTrue(not mod1.exists() and
                        self.project.find_module('newmod') is not None)
        self.assertEqual('from newmod import a_func\n', mod2.read())

    def test_renaming_modules_aliased(self):
        mod1 = testutils.create_module(self.project, 'mod1')
        mod1.write('def a_func():\n    pass\n')
        mod2 = testutils.create_module(self.project, 'mod2')
        mod2.write('import mod1 as m\nm.a_func()\n')
        self._rename(mod1, None, 'newmod')
        self.assertTrue(not mod1.exists() and
                        self.project.find_module('newmod') is not None)
        self.assertEqual('import newmod as m\nm.a_func()\n', mod2.read())

    def test_renaming_packages(self):
        pkg = testutils.create_package(self.project, 'pkg')
        mod1 = testutils.create_module(self.project, 'mod1', pkg)
        mod1.write('def a_func():\n    pass\n')
        mod2 = testutils.create_module(self.project, 'mod2', pkg)
        mod2.write('from pkg.mod1 import a_func\n')
        self._rename(mod2, 6, 'newpkg')
        self.assertTrue(self.project.find_module('newpkg.mod1') is not None)
        new_mod2 = self.project.find_module('newpkg.mod2')
        self.assertEqual('from newpkg.mod1 import a_func\n', new_mod2.read())

    def test_module_dependencies(self):
        mod1 = testutils.create_module(self.project, 'mod1')
        mod1.write('class AClass(object):\n    pass\n')
        mod2 = testutils.create_module(self.project, 'mod2')
        mod2.write('import mod1\na_var = mod1.AClass()\n')
        self.project.get_pymodule(mod2).get_attributes()['mod1']
        mod1.write('def AClass():\n    return 0\n')

        self._rename(mod2, len(mod2.read()) - 3, 'a_func')
        self.assertEqual('def a_func():\n    return 0\n', mod1.read())
        self.assertEqual('import mod1\na_var = mod1.a_func()\n', mod2.read())

    def test_renaming_class_attributes(self):
        mod1 = testutils.create_module(self.project, 'mod1')
        mod1.write('class AClass(object):\n    def __init__(self):\n'
                   '        self.an_attr = 10\n')
        mod2 = testutils.create_module(self.project, 'mod2')
        mod2.write('import mod1\na_var = mod1.AClass()\n'
                   'another_var = a_var.an_attr')

        self._rename(mod1, mod1.read().index('an_attr'), 'attr')
        self.assertEqual('class AClass(object):\n    def __init__(self):\n'
                          '        self.attr = 10\n', mod1.read())
        self.assertEqual(
            'import mod1\na_var = mod1.AClass()\nanother_var = a_var.attr',
            mod2.read())

    def test_renaming_class_attributes2(self):
        mod1 = testutils.create_module(self.project, 'mod1')
        mod1.write('class AClass(object):\n    def __init__(self):\n'
                   '        an_attr = 10\n        self.an_attr = 10\n')
        mod2 = testutils.create_module(self.project, 'mod2')
        mod2.write('import mod1\na_var = mod1.AClass()\n'
                   'another_var = a_var.an_attr')

        self._rename(mod1, mod1.read().rindex('an_attr'), 'attr')
        self.assertEqual(
            'class AClass(object):\n    def __init__(self):\n'
            '        an_attr = 10\n        self.attr = 10\n', mod1.read())
        self.assertEqual(
            'import mod1\na_var = mod1.AClass()\nanother_var = a_var.attr',
            mod2.read())

    def test_renaming_methods_in_subclasses(self):
        mod = testutils.create_module(self.project, 'mod1')
        mod.write('class A(object):\n    def a_method(self):\n        pass\n'
                  'class B(A):\n    def a_method(self):\n        pass\n')

        self._rename(mod, mod.read().rindex('a_method') + 1, 'new_method',
                     in_hierarchy=True)
        self.assertEqual(
            'class A(object):\n    def new_method(self):\n        pass\n'
            'class B(A):\n    def new_method(self):\n        pass\n',
            mod.read())

    def test_renaming_methods_in_sibling_classes(self):
        mod = testutils.create_module(self.project, 'mod1')
        mod.write('class A(object):\n    def a_method(self):\n        pass\n'
                  'class B(A):\n    def a_method(self):\n        pass\n'
                  'class C(A):\n    def a_method(self):\n        pass\n')

        self._rename(mod, mod.read().rindex('a_method') + 1, 'new_method',
                     in_hierarchy=True)
        self.assertEqual(
            'class A(object):\n    def new_method(self):\n        pass\n'
            'class B(A):\n    def new_method(self):\n        pass\n'
            'class C(A):\n    def new_method(self):\n        pass\n',
            mod.read())

    def test_not_renaming_methods_in_hierarchies(self):
        mod = testutils.create_module(self.project, 'mod1')
        mod.write('class A(object):\n    def a_method(self):\n        pass\n'
                  'class B(A):\n    def a_method(self):\n        pass\n')

        self._rename(mod, mod.read().rindex('a_method') + 1, 'new_method',
                     in_hierarchy=False)
        self.assertEqual(
            'class A(object):\n    def a_method(self):\n        pass\n'
            'class B(A):\n    def new_method(self):\n        pass\n',
            mod.read())

    def test_undoing_refactorings(self):
        mod1 = testutils.create_module(self.project, 'mod1')
        mod1.write('def a_func():\n    pass\na_func()\n')
        self._rename(mod1, len(mod1.read()) - 5, 'new_func')
        self.project.history.undo()
        self.assertEqual('def a_func():\n    pass\na_func()\n', mod1.read())

    def test_undoing_renaming_modules(self):
        mod1 = testutils.create_module(self.project, 'mod1')
        mod1.write('def a_func():\n    pass\n')
        mod2 = testutils.create_module(self.project, 'mod2')
        mod2.write('from mod1 import a_func\n')
        self._rename(mod2, 6, 'newmod')
        self.project.history.undo()
        self.assertEqual('mod1.py', mod1.path)
        self.assertEqual('from mod1 import a_func\n', mod2.read())

    def test_rename_in_module_renaming_one_letter_names_for_expressions(self):
        mod1 = testutils.create_module(self.project, 'mod1')
        mod1.write('a = 10\nprint(1+a)\n')
        pymod = self.project.get_module('mod1')
        old_pyname = pymod['a']
        finder = rope.refactor.occurrences.create_finder(
            self.project, 'a', old_pyname)
        refactored = rename.rename_in_module(
            finder, 'new_var', pymodule=pymod, replace_primary=True)
        self.assertEqual('new_var = 10\nprint(1+new_var)\n', refactored)

    def test_renaming_for_loop_variable(self):
        code = 'for var in range(10):\n    print(var)\n'
        refactored = self._local_rename(code, code.find('var') + 1, 'new_var')
        self.assertEqual('for new_var in range(10):\n    print(new_var)\n',
                          refactored)

    @testutils.only_for('3.5')
    def test_renaming_async_for_loop_variable(self):
        code = 'async def func():\n    async for var in range(10):\n        print(var)\n'
        refactored = self._local_rename(code, code.find('var') + 1, 'new_var')
        self.assertEqual('async def func():\n    async for new_var in range(10):\n        print(new_var)\n',
                          refactored)

    @testutils.only_for('3.5')
    def test_renaming_async_with_context_manager(self):
        code = 'def a_cm(): pass\n'\
               'async def a_func():\n    async with a_cm() as x: pass'
        refactored = self._local_rename(code, code.find('a_cm') + 1, 'another_cm')
        expected = 'def another_cm(): pass\n'\
                   'async def a_func():\n    async with another_cm() as x: pass'
        self.assertEqual(refactored, expected)

    @testutils.only_for('3.5')
    def test_renaming_async_with_as_variable(self):
        code = 'async def func():\n    async with a_func() as var:\n        print(var)\n'
        refactored = self._local_rename(code, code.find('var') + 1, 'new_var')
        self.assertEqual('async def func():\n    async with a_func() as new_var:\n        print(new_var)\n',
                          refactored)

    def test_renaming_parameters(self):
        code = 'def a_func(param):\n    print(param)\na_func(param=hey)\n'
        refactored = self._local_rename(code, code.find('param') + 1,
                                        'new_param')
        self.assertEqual('def a_func(new_param):\n    print(new_param)\n'
                          'a_func(new_param=hey)\n', refactored)

    def test_renaming_assigned_parameters(self):
        code = 'def f(p):\n    p = p + 1\n    return p\nf(p=1)\n'
        refactored = self._local_rename(code, code.find('p'), 'arg')
        self.assertEqual('def f(arg):\n    arg = arg + 1\n'
                          '    return arg\nf(arg=1)\n', refactored)

    def test_renaming_parameters_not_renaming_others(self):
        code = 'def a_func(param):' \
            '\n    print(param)\nparam=10\na_func(param)\n'
        refactored = self._local_rename(code, code.find('param') + 1,
                                        'new_param')
        self.assertEqual('def a_func(new_param):\n    print(new_param)\n'
                          'param=10\na_func(param)\n', refactored)

    def test_renaming_parameters_not_renaming_others2(self):
        code = 'def a_func(param):\n    print(param)\n' \
            'param=10\na_func(param=param)'
        refactored = self._local_rename(code, code.find('param') + 1,
                                        'new_param')
        self.assertEqual('def a_func(new_param):\n    print(new_param)\n'
                          'param=10\na_func(new_param=param)', refactored)

    def test_renaming_parameters_with_multiple_params(self):
        code = 'def a_func(param1, param2):\n    print(param1)\n'\
               'a_func(param1=1, param2=2)\n'
        refactored = self._local_rename(code, code.find('param1') + 1,
                                        'new_param')
        self.assertEqual(
            'def a_func(new_param, param2):\n    print(new_param)\n'
            'a_func(new_param=1, param2=2)\n', refactored)

    def test_renaming_parameters_with_multiple_params2(self):
        code = 'def a_func(param1, param2):\n    print(param1)\n' \
               'a_func(param1=1, param2=2)\n'
        refactored = self._local_rename(code, code.rfind('param2') + 1,
                                        'new_param')
        self.assertEqual('def a_func(param1, new_param):\n    print(param1)\n'
                          'a_func(param1=1, new_param=2)\n', refactored)

    def test_renaming_parameters_on_calls(self):
        code = 'def a_func(param):\n    print(param)\na_func(param = hey)\n'
        refactored = self._local_rename(code, code.rfind('param') + 1,
                                        'new_param')
        self.assertEqual('def a_func(new_param):\n    print(new_param)\n'
                          'a_func(new_param = hey)\n', refactored)

    def test_renaming_parameters_spaces_before_call(self):
        code = 'def a_func(param):\n    print(param)\na_func  (param=hey)\n'
        refactored = self._local_rename(code, code.rfind('param') + 1,
                                        'new_param')
        self.assertEqual('def a_func(new_param):\n    print(new_param)\n'
                          'a_func  (new_param=hey)\n', refactored)

    def test_renaming_parameter_like_objects_after_keywords(self):
        code = 'def a_func(param):\n    print(param)\ndict(param=hey)\n'
        refactored = self._local_rename(code, code.find('param') + 1,
                                        'new_param')
        self.assertEqual('def a_func(new_param):\n    print(new_param)\n'
                          'dict(param=hey)\n', refactored)

    def test_renaming_variables_in_init_dot_pys(self):
        pkg = testutils.create_package(self.project, 'pkg')
        init_dot_py = pkg.get_child('__init__.py')
        init_dot_py.write('a_var = 10\n')
        mod = testutils.create_module(self.project, 'mod')
        mod.write('import pkg\nprint(pkg.a_var)\n')
        self._rename(mod, mod.read().index('a_var') + 1, 'new_var')
        self.assertEqual('new_var = 10\n', init_dot_py.read())
        self.assertEqual('import pkg\nprint(pkg.new_var)\n', mod.read())

    def test_renaming_variables_in_init_dot_pys2(self):
        pkg = testutils.create_package(self.project, 'pkg')
        init_dot_py = pkg.get_child('__init__.py')
        init_dot_py.write('a_var = 10\n')
        mod = testutils.create_module(self.project, 'mod')
        mod.write('import pkg\nprint(pkg.a_var)\n')
        self._rename(init_dot_py,
                     init_dot_py.read().index('a_var') + 1, 'new_var')
        self.assertEqual('new_var = 10\n', init_dot_py.read())
        self.assertEqual('import pkg\nprint(pkg.new_var)\n', mod.read())

    def test_renaming_variables_in_init_dot_pys3(self):
        pkg = testutils.create_package(self.project, 'pkg')
        init_dot_py = pkg.get_child('__init__.py')
        init_dot_py.write('a_var = 10\n')
        mod = testutils.create_module(self.project, 'mod')
        mod.write('import pkg\nprint(pkg.a_var)\n')
        self._rename(mod, mod.read().index('a_var') + 1, 'new_var')
        self.assertEqual('new_var = 10\n', init_dot_py.read())
        self.assertEqual('import pkg\nprint(pkg.new_var)\n', mod.read())

    def test_renaming_resources_using_rename_module_refactoring(self):
        mod1 = testutils.create_module(self.project, 'mod1')
        mod2 = testutils.create_module(self.project, 'mod2')
        mod1.write('a_var = 1')
        mod2.write('import mod1\nmy_var = mod1.a_var\n')
        renamer = rename.Rename(self.project, mod1)
        renamer.get_changes('newmod').do()
        self.assertEqual('import newmod\nmy_var = newmod.a_var\n',
                          mod2.read())

    def test_renam_resources_using_rename_module_refactor_for_packages(self):
        mod1 = testutils.create_module(self.project, 'mod1')
        pkg = testutils.create_package(self.project, 'pkg')
        mod1.write('import pkg\nmy_pkg = pkg')
        renamer = rename.Rename(self.project, pkg)
        renamer.get_changes('newpkg').do()
        self.assertEqual('import newpkg\nmy_pkg = newpkg', mod1.read())

    def test_renam_resources_use_rename_module_refactor_for_init_dot_py(self):
        mod1 = testutils.create_module(self.project, 'mod1')
        pkg = testutils.create_package(self.project, 'pkg')
        mod1.write('import pkg\nmy_pkg = pkg')
        renamer = rename.Rename(self.project, pkg.get_child('__init__.py'))
        renamer.get_changes('newpkg').do()
        self.assertEqual('import newpkg\nmy_pkg = newpkg', mod1.read())

    def test_renaming_global_variables(self):
        code = 'a_var = 1\ndef a_func():\n    global a_var\n    var = a_var\n'
        refactored = self._local_rename(code, code.index('a_var'), 'new_var')
        self.assertEqual(
            'new_var = 1\ndef a_func():\n    '
            'global new_var\n    var = new_var\n',
            refactored)

    def test_renaming_global_variables2(self):
        code = 'a_var = 1\ndef a_func():\n    global a_var\n    var = a_var\n'
        refactored = self._local_rename(code, code.rindex('a_var'), 'new_var')
        self.assertEqual(
            'new_var = 1\ndef a_func():\n    '
            'global new_var\n    var = new_var\n',
            refactored)

    def test_renaming_when_unsure(self):
        code = 'class C(object):\n    def a_func(self):\n        pass\n' \
               'def f(arg):\n    arg.a_func()\n'
        mod1 = testutils.create_module(self.project, 'mod1')
        mod1.write(code)
        self._rename(mod1, code.index('a_func'),
                     'new_func', unsure=self._true)
        self.assertEqual(
            'class C(object):\n    def new_func(self):\n        pass\n'
            'def f(arg):\n    arg.new_func()\n',
            mod1.read())

    def _true(self, *args):
        return True

    def test_renaming_when_unsure_with_confirmation(self):
        def confirm(occurrence):
            return False
        code = 'class C(object):\n    def a_func(self):\n        pass\n' \
               'def f(arg):\n    arg.a_func()\n'
        mod1 = testutils.create_module(self.project, 'mod1')
        mod1.write(code)
        self._rename(mod1, code.index('a_func'), 'new_func', unsure=confirm)
        self.assertEqual(
            'class C(object):\n    def new_func(self):\n        pass\n'
            'def f(arg):\n    arg.a_func()\n', mod1.read())

    def test_renaming_when_unsure_not_renaming_knowns(self):
        code = 'class C1(object):\n    def a_func(self):\n        pass\n' \
               'class C2(object):\n    def a_func(self):\n        pass\n' \
               'c1 = C1()\nc1.a_func()\nc2 = C2()\nc2.a_func()\n'
        mod1 = testutils.create_module(self.project, 'mod1')
        mod1.write(code)
        self._rename(mod1, code.index('a_func'), 'new_func', unsure=self._true)
        self.assertEqual(
            'class C1(object):\n    def new_func(self):\n        pass\n'
            'class C2(object):\n    def a_func(self):\n        pass\n'
            'c1 = C1()\nc1.new_func()\nc2 = C2()\nc2.a_func()\n',
            mod1.read())

    def test_renaming_in_strings_and_comments(self):
        code = 'a_var = 1\n# a_var\n'
        mod1 = testutils.create_module(self.project, 'mod1')
        mod1.write(code)
        self._rename(mod1, code.index('a_var'), 'new_var', docs=True)
        self.assertEqual('new_var = 1\n# new_var\n', mod1.read())

    def test_not_renaming_in_strings_and_comments_where_not_visible(self):
        code = 'def f():\n    a_var = 1\n# a_var\n'
        mod1 = testutils.create_module(self.project, 'mod1')
        mod1.write(code)
        self._rename(mod1, code.index('a_var'), 'new_var', docs=True)
        self.assertEqual('def f():\n    new_var = 1\n# a_var\n', mod1.read())

    def test_not_renaming_all_text_occurrences_in_strings_and_comments(self):
        code = 'a_var = 1\n# a_vard _a_var\n'
        mod1 = testutils.create_module(self.project, 'mod1')
        mod1.write(code)
        self._rename(mod1, code.index('a_var'), 'new_var', docs=True)
        self.assertEqual('new_var = 1\n# a_vard _a_var\n', mod1.read())

    def test_renaming_occurrences_in_overwritten_scopes(self):
        refactored = self._local_rename(
            'a_var = 20\ndef f():\n    print(a_var)\n'
            'def f():\n    print(a_var)\n', 2, 'new_var')
        self.assertEqual('new_var = 20\ndef f():\n    print(new_var)\n'
                          'def f():\n    print(new_var)\n', refactored)

    def test_renaming_occurrences_in_overwritten_scopes2(self):
        code = 'def f():\n    a_var = 1\n    print(a_var)\n' \
               'def f():\n    a_var = 1\n    print(a_var)\n'
        refactored = self._local_rename(code, code.index('a_var') + 1,
                                        'new_var')
        self.assertEqual(code.replace('a_var', 'new_var', 2), refactored)

    @testutils.only_for_versions_higher('3.5')
    def test_renaming_in_generalized_dict_unpacking(self):
        code = dedent('''\
            a_var = {**{'stuff': 'can'}, **{'stuff': 'crayon'}}

            if "stuff" in a_var:
                print("ya")
        ''')
        mod1 = testutils.create_module(self.project, 'mod1')
        mod1.write(code)
        refactored = self._local_rename(code, code.index('a_var') + 1,
                                        'new_var')
        expected = dedent('''\
            new_var = {**{'stuff': 'can'}, **{'stuff': 'crayon'}}

            if "stuff" in new_var:
                print("ya")
        ''')
        self.assertEqual(expected, refactored)

    def test_dos_line_ending_and_renaming(self):
        code = '\r\na = 1\r\n\r\nprint(2 + a + 2)\r\n'
        offset = code.replace('\r\n', '\n').rindex('a')
        refactored = self._local_rename(code, offset, 'b')
        self.assertEqual('\nb = 1\n\nprint(2 + b + 2)\n',
                          refactored.replace('\r\n', '\n'))

    def test_multi_byte_strs_and_renaming(self):
        s = u'{LATIN SMALL LETTER I WITH DIAERESIS}' * 4
        code = u'# -*- coding: utf-8 -*-\n# ' + s + \
            '\na = 1\nprint(2 + a + 2)\n'
        refactored = self._local_rename(code, code.rindex('a'), 'b')
        self.assertEqual(u'# -*- coding: utf-8 -*-\n# ' + s +
                          '\nb = 1\nprint(2 + b + 2)\n', refactored)

    def test_resources_parameter(self):
        mod1 = testutils.create_module(self.project, 'mod1')
        mod2 = testutils.create_module(self.project, 'mod2')
        mod1.write('def f():\n    pass\n')
        mod2.write('import mod1\nmod1.f()\n')
        self._rename(mod1, mod1.read().rindex('f'), 'g',
                     resources=[mod1])
        self.assertEqual('def g():\n    pass\n', mod1.read())
        self.assertEqual('import mod1\nmod1.f()\n', mod2.read())

    def test_resources_parameter_not_changing_defining_module(self):
        mod1 = testutils.create_module(self.project, 'mod1')
        mod2 = testutils.create_module(self.project, 'mod2')
        mod1.write('def f():\n    pass\n')
        mod2.write('import mod1\nmod1.f()\n')
        self._rename(mod1, mod1.read().rindex('f'), 'g',
                     resources=[mod2])
        self.assertEqual('def f():\n    pass\n', mod1.read())
        self.assertEqual('import mod1\nmod1.g()\n', mod2.read())

    # XXX: with variables should not leak
    @testutils.only_for('2.5')
    def xxx_test_with_statement_variables_should_not_leak(self):
        code = 'f = 1\nwith open("1.txt") as f:\n    print(f)\n'
        if sys.version_info < (2, 6, 0):
            code = 'from __future__ import with_statement\n' + code
        mod1 = testutils.create_module(self.project, 'mod1')
        mod1.write(code)
        self._rename(mod1, code.rindex('f'), 'file')
        expected = 'f = 1\nwith open("1.txt") as file:\n    print(file)\n'
        self.assertEqual(expected, mod1.read())


class ChangeOccurrencesTest(unittest.TestCase):

    def setUp(self):
        self.project = testutils.sample_project()
        self.mod = testutils.create_module(self.project, 'mod')

    def tearDown(self):
        testutils.remove_project(self.project)
        super(ChangeOccurrencesTest, self).tearDown()

    def test_simple_case(self):
        self.mod.write('a_var = 1\nprint(a_var)\n')
        changer = rename.ChangeOccurrences(self.project, self.mod,
                                           self.mod.read().index('a_var'))
        changer.get_changes('new_var').do()
        self.assertEqual('new_var = 1\nprint(new_var)\n', self.mod.read())

    def test_only_performing_inside_scopes(self):
        self.mod.write('a_var = 1\nnew_var = 2\ndef f():\n    print(a_var)\n')
        changer = rename.ChangeOccurrences(self.project, self.mod,
                                           self.mod.read().rindex('a_var'))
        changer.get_changes('new_var').do()
        self.assertEqual(
            'a_var = 1\nnew_var = 2\ndef f():\n    print(new_var)\n',
            self.mod.read())

    def test_only_performing_on_calls(self):
        self.mod.write('def f1():\n    pass\ndef f2():\n    pass\n'
                       'g = f1\na = f1()\n')
        changer = rename.ChangeOccurrences(self.project, self.mod,
                                           self.mod.read().rindex('f1'))
        changer.get_changes('f2', only_calls=True).do()
        self.assertEqual(
            'def f1():\n    pass\ndef f2():\n    pass\ng = f1\na = f2()\n',
            self.mod.read())

    def test_only_performing_on_reads(self):
        self.mod.write('a = 1\nb = 2\nprint(a)\n')
        changer = rename.ChangeOccurrences(self.project, self.mod,
                                           self.mod.read().rindex('a'))
        changer.get_changes('b', writes=False).do()
        self.assertEqual('a = 1\nb = 2\nprint(b)\n', self.mod.read())


class ImplicitInterfacesTest(unittest.TestCase):

    def setUp(self):
        super(ImplicitInterfacesTest, self).setUp()
        self.project = testutils.sample_project(validate_objectdb=True)
        self.pycore = self.project.pycore
        self.mod1 = testutils.create_module(self.project, 'mod1')
        self.mod2 = testutils.create_module(self.project, 'mod2')

    def tearDown(self):
        testutils.remove_project(self.project)
        super(ImplicitInterfacesTest, self).tearDown()

    def _rename(self, resource, offset, new_name, **kwds):
        changes = Rename(self.project, resource, offset).\
            get_changes(new_name, **kwds)
        self.project.do(changes)

    def test_performing_rename_on_parameters(self):
        self.mod1.write('def f(arg):\n    arg.run()\n')
        self.mod2.write('import mod1\n\n\n'
                        'class A(object):\n    def run(self):\n        pass\n'
                        'class B(object):\n    def run(self):\n        pass\n'
                        'mod1.f(A())\nmod1.f(B())\n')
        self.pycore.analyze_module(self.mod2)
        self._rename(self.mod1, self.mod1.read().index('run'), 'newrun')
        self.assertEqual('def f(arg):\n    arg.newrun()\n', self.mod1.read())
        self.assertEqual(
            'import mod1\n\n\n'
            'class A(object):\n    def newrun(self):\n        pass\n'
            'class B(object):\n    def newrun(self):\n        pass\n'
            'mod1.f(A())\nmod1.f(B())\n', self.mod2.read())


def suite():
    result = unittest.TestSuite()
    result.addTests(unittest.makeSuite(RenameRefactoringTest))
    result.addTests(unittest.makeSuite(ChangeOccurrencesTest))
    result.addTests(unittest.makeSuite(ImplicitInterfacesTest))
    return result


if __name__ == '__main__':
    unittest.main()



try:
    import unittest2 as unittest
except ImportError:
    import unittest

from rope.refactor import similarfinder
from ropetest import testutils


class SimilarFinderTest(unittest.TestCase):

    def setUp(self):
        super(SimilarFinderTest, self).setUp()
        self.project = testutils.sample_project()
        self.mod = testutils.create_module(self.project, 'mod')

    def tearDown(self):
        testutils.remove_project(self.project)
        super(SimilarFinderTest, self).tearDown()

    def _create_finder(self, source, **kwds):
        self.mod.write(source)
        pymodule = self.project.get_pymodule(self.mod)
        return similarfinder.SimilarFinder(pymodule, **kwds)

    def test_trivial_case(self):
        finder = self._create_finder('')
        self.assertEqual([], list(finder.get_match_regions('10')))

    def test_constant_integer(self):
        source = 'a = 10\n'
        finder = self._create_finder(source)
        result = [(source.index('10'), source.index('10') + 2)]
        self.assertEqual(result, list(finder.get_match_regions('10')))

    def test_bool_is_not_similar_to_integer(self):
        source = 'a = False\nb = 0'
        finder = self._create_finder(source)
        result = [(source.index('False'), source.index('False') + len('False'))]
        self.assertEqual(result, list(finder.get_match_regions('False')))

    def test_simple_addition(self):
        source = 'a = 1 + 2\n'
        finder = self._create_finder(source)
        result = [(source.index('1'), source.index('2') + 1)]
        self.assertEqual(result, list(finder.get_match_regions('1 + 2')))

    def test_simple_addition2(self):
        source = 'a = 1 +2\n'
        finder = self._create_finder(source)
        result = [(source.index('1'), source.index('2') + 1)]
        self.assertEqual(result, list(finder.get_match_regions('1 + 2')))

    def test_simple_assign_statements(self):
        source = 'a = 1 + 2\n'
        finder = self._create_finder(source)
        self.assertEqual([(0, len(source) - 1)],
                          list(finder.get_match_regions('a = 1 + 2')))

    def test_simple_multiline_statements(self):
        source = 'a = 1\nb = 2\n'
        finder = self._create_finder(source)
        self.assertEqual([(0, len(source) - 1)],
                          list(finder.get_match_regions('a = 1\nb = 2')))

    def test_multiple_matches(self):
        source = 'a = 1 + 1\n'
        finder = self._create_finder(source)
        result = list(finder.get_match_regions('1'))
        self.assertEqual(2, len(result))
        start1 = source.index('1')
        self.assertEqual((start1, start1 + 1), result[0])
        start2 = source.rindex('1')
        self.assertEqual((start2, start2 + 1), result[1])

    def test_multiple_matches2(self):
        source = 'a = 1\nb = 2\n\na = 1\nb = 2\n'
        finder = self._create_finder(source)
        self.assertEqual(
            2, len(list(finder.get_match_regions('a = 1\nb = 2'))))

    def test_restricting_the_region_to_search(self):
        source = '1\n\n1\n'
        finder = self._create_finder(source)
        result = list(finder.get_match_regions('1', start=2))
        start = source.rfind('1')
        self.assertEqual([(start, start + 1)], result)

    def test_matching_basic_patterns(self):
        source = 'b = a\n'
        finder = self._create_finder(source)
        result = list(finder.get_match_regions('${a}', args={'a': 'exact'}))
        start = source.rfind('a')
        self.assertEqual([(start, start + 1)], result)

    def test_match_get_ast(self):
        source = 'b = a\n'
        finder = self._create_finder(source)
        result = list(finder.get_matches('${a}', args={'a': 'exact'}))
        self.assertEqual('a', result[0].get_ast('a').id)

    def test_match_get_ast_for_statements(self):
        source = 'b = a\n'
        finder = self._create_finder(source)
        result = list(finder.get_matches('b = ${a}'))
        self.assertEqual('a', result[0].get_ast('a').id)

    def test_matching_multiple_patterns(self):
        source = 'c = a + b\n'
        finder = self._create_finder(source)
        result = list(finder.get_matches('${a} + ${b}'))
        self.assertEqual('a', result[0].get_ast('a').id)
        self.assertEqual('b', result[0].get_ast('b').id)

    def test_matching_any_patterns(self):
        source = 'b = a\n'
        finder = self._create_finder(source)
        result = list(finder.get_matches('b = ${x}'))
        self.assertEqual('a', result[0].get_ast('x').id)

    def test_matching_any_patterns_repeating(self):
        source = 'b = 1 + 1\n'
        finder = self._create_finder(source)
        result = list(finder.get_matches('b = ${x} + ${x}'))
        self.assertEqual(1, result[0].get_ast('x').n)

    def test_matching_any_patterns_not_matching_different_nodes(self):
        source = 'b = 1 + 2\n'
        finder = self._create_finder(source)
        result = list(finder.get_matches('b = ${x} + ${x}'))
        self.assertEqual(0, len(result))

    def test_matching_normal_names_and_assname(self):
        source = 'a = 1\n'
        finder = self._create_finder(source)
        result = list(finder.get_matches('${a} = 1'))
        self.assertEqual('a', result[0].get_ast('a').id)

    def test_matching_normal_names_and_assname2(self):
        source = 'a = 1\n'
        finder = self._create_finder(source)
        result = list(finder.get_matches('${a}', args={'a': 'exact'}))
        self.assertEqual(1, len(result))

    def test_matching_normal_names_and_attributes(self):
        source = 'x.a = 1\n'
        finder = self._create_finder(source)
        result = list(finder.get_matches('${a} = 1', args={'a': 'exact'}))
        self.assertEqual(0, len(result))

    def test_functions_not_matching_when_only_first_parameters(self):
        source = 'f(1, 2)\n'
        finder = self._create_finder(source)
        self.assertEqual(0, len(list(finder.get_matches('f(1)'))))

    def test_matching_nested_try_finally(self):
        source = 'if 1:\n    try:\n        pass\n    except:\n        pass\n'
        pattern = 'try:\n    pass\nexcept:\n    pass\n'
        finder = self._create_finder(source)
        self.assertEqual(1, len(list(finder.get_matches(pattern))))

    def test_matching_dicts_inside_functions(self):
        source = 'def f(p):\n    d = {1: p.x}\n'
        pattern = '{1: ${a}.x}'
        finder = self._create_finder(source)
        self.assertEqual(1, len(list(finder.get_matches(pattern))))


class CheckingFinderTest(unittest.TestCase):

    def setUp(self):
        super(CheckingFinderTest, self).setUp()
        self.project = testutils.sample_project()
        self.mod1 = testutils.create_module(self.project, 'mod1')

    def tearDown(self):
        testutils.remove_project(self.project)
        super(CheckingFinderTest, self).tearDown()

    def test_trivial_case(self):
        self.mod1.write('')
        pymodule = self.project.get_pymodule(self.mod1)
        finder = similarfinder.SimilarFinder(pymodule)
        self.assertEqual([], list(finder.get_matches('10', {})))

    def test_simple_finding(self):
        self.mod1.write('class A(object):\n    pass\na = A()\n')
        pymodule = self.project.get_pymodule(self.mod1)
        finder = similarfinder.SimilarFinder(pymodule)
        result = list(finder.get_matches('${anything} = ${A}()', {}))
        self.assertEqual(1, len(result))

    def test_not_matching_when_the_name_does_not_match(self):
        self.mod1.write('class A(object):\n    pass\na = list()\n')
        pymodule = self.project.get_pymodule(self.mod1)
        finder = similarfinder.SimilarFinder(pymodule)
        result = list(finder.get_matches('${anything} = ${C}()',
                                         {'C': 'name=mod1.A'}))
        self.assertEqual(0, len(result))

    def test_not_matching_unknowns_finding(self):
        self.mod1.write('class A(object):\n    pass\na = unknown()\n')
        pymodule = self.project.get_pymodule(self.mod1)
        finder = similarfinder.SimilarFinder(pymodule)
        result = list(finder.get_matches('${anything} = ${C}()',
                                         {'C': 'name=mod1.A'}))
        self.assertEqual(0, len(result))

    def test_finding_and_matching_pyobjects(self):
        source = 'class A(object):\n    pass\nNewA = A\na = NewA()\n'
        self.mod1.write(source)
        pymodule = self.project.get_pymodule(self.mod1)
        finder = similarfinder.SimilarFinder(pymodule)
        result = list(finder.get_matches('${anything} = ${A}()',
                                         {'A': 'object=mod1.A'}))
        self.assertEqual(1, len(result))
        start = source.rindex('a =')
        self.assertEqual((start, len(source) - 1), result[0].get_region())

    def test_finding_and_matching_types(self):
        source = 'class A(object):\n    def f(self):\n        pass\n' \
                 'a = A()\nb = a.f()\n'
        self.mod1.write(source)
        pymodule = self.project.get_pymodule(self.mod1)
        finder = similarfinder.SimilarFinder(pymodule)
        result = list(finder.get_matches('${anything} = ${inst}.f()',
                                         {'inst': 'type=mod1.A'}))
        self.assertEqual(1, len(result))
        start = source.rindex('b')
        self.assertEqual((start, len(source) - 1), result[0].get_region())

    def test_checking_the_type_of_an_ass_name_node(self):
        self.mod1.write('class A(object):\n    pass\nan_a = A()\n')
        pymodule = self.project.get_pymodule(self.mod1)
        finder = similarfinder.SimilarFinder(pymodule)
        result = list(finder.get_matches('${a} = ${assigned}',
                                         {'a': 'type=mod1.A'}))
        self.assertEqual(1, len(result))

    def test_checking_instance_of_an_ass_name_node(self):
        self.mod1.write('class A(object):\n    pass\n'
                        'class B(A):\n    pass\nb = B()\n')
        pymodule = self.project.get_pymodule(self.mod1)
        finder = similarfinder.SimilarFinder(pymodule)
        result = list(finder.get_matches('${a} = ${assigned}',
                                         {'a': 'instance=mod1.A'}))
        self.assertEqual(1, len(result))

    def test_checking_equality_of_imported_pynames(self):
        mod2 = testutils.create_module(self.project, 'mod2')
        mod2.write('class A(object):\n    pass\n')
        self.mod1.write('from mod2 import A\nan_a = A()\n')
        pymod1 = self.project.get_pymodule(self.mod1)
        finder = similarfinder.SimilarFinder(pymod1)
        result = list(finder.get_matches('${a_class}()',
                                         {'a_class': 'name=mod2.A'}))
        self.assertEqual(1, len(result))


class TemplateTest(unittest.TestCase):

    def test_simple_templates(self):
        template = similarfinder.CodeTemplate('${a}\n')
        self.assertEqual(set(['a']), set(template.get_names()))

    def test_ignoring_matches_in_comments(self):
        template = similarfinder.CodeTemplate('#${a}\n')
        self.assertEqual({}.keys(), template.get_names())

    def test_ignoring_matches_in_strings(self):
        template = similarfinder.CodeTemplate("'${a}'\n")
        self.assertEqual({}.keys(), template.get_names())

    def test_simple_substitution(self):
        template = similarfinder.CodeTemplate('${a}\n')
        self.assertEqual('b\n', template.substitute({'a': 'b'}))

    def test_substituting_multiple_names(self):
        template = similarfinder.CodeTemplate('${a}, ${b}\n')
        self.assertEqual('1, 2\n', template.substitute({'a': '1', 'b': '2'}))


def suite():
    result = unittest.TestSuite()
    result.addTests(unittest.makeSuite(SimilarFinderTest))
    result.addTests(unittest.makeSuite(CheckingFinderTest))
    result.addTests(unittest.makeSuite(TemplateTest))
    return result

if __name__ == '__main__':
    unittest.main()



try:
    import unittest2 as unittest
except ImportError:
    import unittest

from rope.base import exceptions
from rope.contrib.findit import (find_occurrences, find_implementations,
                                 find_definition)
from ropetest import testutils


class FindItTest(unittest.TestCase):

    def setUp(self):
        super(FindItTest, self).setUp()
        self.project = testutils.sample_project()

    def tearDown(self):
        testutils.remove_project(self.project)
        super(FindItTest, self).tearDown()

    def test_finding_occurrences(self):
        mod = testutils.create_module(self.project, 'mod')
        mod.write('a_var = 1\n')
        result = find_occurrences(self.project, mod, 1)
        self.assertEqual(mod, result[0].resource)
        self.assertEqual(0, result[0].offset)
        self.assertEqual(False, result[0].unsure)

    def test_finding_occurrences_in_more_than_one_module(self):
        mod1 = testutils.create_module(self.project, 'mod1')
        mod2 = testutils.create_module(self.project, 'mod2')
        mod1.write('a_var = 1\n')
        mod2.write('import mod1\nmy_var = mod1.a_var')
        result = find_occurrences(self.project, mod1, 1)
        self.assertEqual(2, len(result))
        modules = (result[0].resource, result[1].resource)
        self.assertTrue(mod1 in modules and mod2 in modules)

    def test_finding_occurrences_matching_when_unsure(self):
        mod1 = testutils.create_module(self.project, 'mod1')
        mod1.write('class C(object):\n    def a_func(self):\n        pass\n'
                   'def f(arg):\n    arg.a_func()\n')
        result = find_occurrences(
            self.project, mod1, mod1.read().index('a_func'), unsure=True)
        self.assertEqual(2, len(result))

    def test_find_occurrences_resources_parameter(self):
        mod1 = testutils.create_module(self.project, 'mod1')
        mod2 = testutils.create_module(self.project, 'mod2')
        mod1.write('a_var = 1\n')
        mod2.write('import mod1\nmy_var = mod1.a_var')
        result = find_occurrences(self.project, mod1, 1, resources=[mod1])
        self.assertEqual(1, len(result))
        self.assertEqual((mod1, 0), (result[0].resource, result[0].offset))

    def test_find_occurrences_and_class_hierarchies(self):
        mod1 = testutils.create_module(self.project, 'mod1')
        mod1.write('class A(object):\n    def f():\n        pass\n'
                   'class B(A):\n    def f():\n        pass\n')
        offset = mod1.read().rindex('f')
        result1 = find_occurrences(self.project, mod1, offset)
        result2 = find_occurrences(self.project, mod1,
                                   offset, in_hierarchy=True)
        self.assertEqual(1, len(result1))
        self.assertEqual(2, len(result2))

    def test_trivial_find_implementations(self):
        mod1 = testutils.create_module(self.project, 'mod1')
        mod1.write('class A(object):\n    def f(self):\n        pass\n')
        offset = mod1.read().rindex('f(')
        result = find_implementations(self.project, mod1, offset)
        self.assertEqual([], result)

    def test_find_implementations_and_not_returning_parents(self):
        mod1 = testutils.create_module(self.project, 'mod1')
        mod1.write('class A(object):\n    def f(self):\n        pass\n'
                   'class B(A):\n    def f(self):\n        pass\n')
        offset = mod1.read().rindex('f(')
        result = find_implementations(self.project, mod1, offset)
        self.assertEqual([], result)

    def test_find_implementations_real_implementation(self):
        mod1 = testutils.create_module(self.project, 'mod1')
        mod1.write('class A(object):\n    def f(self):\n        pass\n'
                   'class B(A):\n    def f(self):\n        pass\n')
        offset = mod1.read().index('f(')
        result = find_implementations(self.project, mod1, offset)
        self.assertEqual(1, len(result))
        self.assertEqual(mod1.read().rindex('f('), result[0].offset)

    def test_find_implementations_real_implementation_simple(self):
        mod1 = testutils.create_module(self.project, 'mod1')
        mod1.write('class A(object):\n    pass\n')
        offset = mod1.read().index('A')
        with self.assertRaises(exceptions.BadIdentifierError):
            find_implementations(self.project, mod1, offset)

    def test_trivial_find_definition(self):
        code = 'def a_func():\n    pass\na_func()'
        result = find_definition(self.project, code, code.rindex('a_func'))
        start = code.index('a_func')
        self.assertEqual(start, result.offset)
        self.assertEqual(None, result.resource)
        self.assertEqual(1, result.lineno)
        self.assertEqual((start, start + len('a_func')), result.region)

    def test_find_definition_in_other_modules(self):
        mod1 = testutils.create_module(self.project, 'mod1')
        mod1.write('var = 1\n')
        code = 'import mod1\nprint(mod1.var)\n'
        result = find_definition(self.project, code, code.index('var'))
        self.assertEqual(mod1, result.resource)
        self.assertEqual(0, result.offset)


def suite():
    result = unittest.TestSuite()
    result.addTests(unittest.makeSuite(FindItTest))
    return result

if __name__ == '__main__':
    unittest.main()



try:
    import unittest2 as unittest
except ImportError:
    import unittest

from textwrap import dedent

from rope.refactor.importutils import ImportTools, importinfo, add_import
from ropetest import testutils


class ImportUtilsTest(unittest.TestCase):

    def setUp(self):
        super(ImportUtilsTest, self).setUp()
        self.project = testutils.sample_project()
        self.import_tools = ImportTools(self.project)

        self.mod = testutils.create_module(self.project, 'mod')
        self.pkg1 = testutils.create_package(self.project, 'pkg1')
        self.mod1 = testutils.create_module(self.project, 'mod1', self.pkg1)
        self.pkg2 = testutils.create_package(self.project, 'pkg2')
        self.mod2 = testutils.create_module(self.project, 'mod2', self.pkg2)
        self.mod3 = testutils.create_module(self.project, 'mod3', self.pkg2)
        p1 = testutils.create_package(self.project, 'p1')
        p2 = testutils.create_package(self.project, 'p2', p1)
        p3 = testutils.create_package(self.project, 'p3', p2)
        m1 = testutils.create_module(self.project, 'm1', p3)  # noqa
        l = testutils.create_module(self.project, 'l', p3)  # noqa

    def tearDown(self):
        testutils.remove_project(self.project)
        super(ImportUtilsTest, self).tearDown()

    def test_get_import_for_module(self):
        mod = self.project.find_module('mod')
        import_statement = self.import_tools.get_import(mod)
        self.assertEqual('import mod',
                          import_statement.get_import_statement())

    def test_get_import_for_module_in_nested_modules(self):
        mod = self.project.find_module('pkg1.mod1')
        import_statement = self.import_tools.get_import(mod)
        self.assertEqual('import pkg1.mod1',
                          import_statement.get_import_statement())

    def test_get_import_for_module_in_init_dot_py(self):
        init_dot_py = self.pkg1.get_child('__init__.py')
        import_statement = self.import_tools.get_import(init_dot_py)
        self.assertEqual('import pkg1',
                          import_statement.get_import_statement())

    def test_get_from_import_for_module(self):
        mod = self.project.find_module('mod')
        import_statement = self.import_tools.get_from_import(mod, 'a_func')
        self.assertEqual('from mod import a_func',
                          import_statement.get_import_statement())

    def test_get_from_import_for_module_in_nested_modules(self):
        mod = self.project.find_module('pkg1.mod1')
        import_statement = self.import_tools.get_from_import(mod, 'a_func')
        self.assertEqual('from pkg1.mod1 import a_func',
                          import_statement.get_import_statement())

    def test_get_from_import_for_module_in_init_dot_py(self):
        init_dot_py = self.pkg1.get_child('__init__.py')
        import_statement = self.import_tools.\
            get_from_import(init_dot_py, 'a_func')
        self.assertEqual('from pkg1 import a_func',
                          import_statement.get_import_statement())

    def test_get_import_statements(self):
        self.mod.write('import pkg1\n')
        pymod = self.project.get_module('mod')
        module_with_imports = self.import_tools.module_imports(pymod)
        imports = module_with_imports.imports
        self.assertEqual('import pkg1',
                          imports[0].import_info.get_import_statement())

    def test_get_import_statements_with_alias(self):
        self.mod.write('import pkg1.mod1 as mod1\n')
        pymod = self.project.get_module('mod')
        module_with_imports = self.import_tools.module_imports(pymod)
        imports = module_with_imports.imports
        self.assertEqual('import pkg1.mod1 as mod1',
                          imports[0].import_info.get_import_statement())

    def test_get_import_statements_for_froms(self):
        self.mod.write('from pkg1 import mod1\n')
        pymod = self.project.get_module('mod')
        module_with_imports = self.import_tools.module_imports(pymod)
        imports = module_with_imports.imports
        self.assertEqual('from pkg1 import mod1',
                          imports[0].import_info.get_import_statement())

    def test_get_multi_line_import_statements_for_froms(self):
        self.mod.write('from pkg1 \\\n    import mod1\n')
        pymod = self.project.get_module('mod')
        module_with_imports = self.import_tools.module_imports(pymod)
        imports = module_with_imports.imports
        self.assertEqual('from pkg1 import mod1',
                          imports[0].import_info.get_import_statement())

    def test_get_import_statements_for_from_star(self):
        self.mod.write('from pkg1 import *\n')
        pymod = self.project.get_module('mod')
        module_with_imports = self.import_tools.module_imports(pymod)
        imports = module_with_imports.imports
        self.assertEqual('from pkg1 import *',
                          imports[0].import_info.get_import_statement())

    @testutils.only_for('2.5')
    def test_get_import_statements_for_new_relatives(self):
        self.mod2.write('from .mod3 import x\n')
        pymod = self.project.get_module('pkg2.mod2')
        module_with_imports = self.import_tools.module_imports(pymod)
        imports = module_with_imports.imports
        self.assertEqual('from .mod3 import x',
                          imports[0].import_info.get_import_statement())

    def test_ignoring_indented_imports(self):
        self.mod.write('if True:\n    import pkg1\n')
        pymod = self.project.get_module('mod')
        module_with_imports = self.import_tools.module_imports(pymod)
        imports = module_with_imports.imports
        self.assertEqual(0, len(imports))

    def test_import_get_names(self):
        self.mod.write('import pkg1 as pkg\n')
        pymod = self.project.get_module('mod')
        module_with_imports = self.import_tools.module_imports(pymod)
        imports = module_with_imports.imports
        context = importinfo.ImportContext(self.project, self.project.root)
        self.assertEqual(['pkg'],
                          imports[0].import_info.get_imported_names(context))

    def test_import_get_names_with_alias(self):
        self.mod.write('import pkg1.mod1\n')
        pymod = self.project.get_module('mod')
        module_with_imports = self.import_tools.module_imports(pymod)
        imports = module_with_imports.imports
        context = importinfo.ImportContext(self.project, self.project.root)
        self.assertEqual(['pkg1'],
                          imports[0].import_info.get_imported_names(context))

    def test_import_get_names_with_alias2(self):
        self.mod1.write('def a_func():\n    pass\n')
        self.mod.write('from pkg1.mod1 import *\n')
        pymod = self.project.get_module('mod')
        module_with_imports = self.import_tools.module_imports(pymod)
        imports = module_with_imports.imports
        context = importinfo.ImportContext(self.project, self.project.root)
        self.assertEqual(['a_func'],
                          imports[0].import_info.get_imported_names(context))

    def test_empty_getting_used_imports(self):
        self.mod.write('')
        pymod = self.project.get_module('mod')
        module_with_imports = self.import_tools.module_imports(pymod)
        imports = module_with_imports.get_used_imports(pymod)
        self.assertEqual(0, len(imports))

    def test_empty_getting_used_imports2(self):
        self.mod.write('import pkg\n')
        pymod = self.project.get_module('mod')
        module_with_imports = self.import_tools.module_imports(pymod)
        imports = module_with_imports.get_used_imports(pymod)
        self.assertEqual(0, len(imports))

    def test_simple_getting_used_imports(self):
        self.mod.write('import pkg\nprint(pkg)\n')
        pymod = self.project.get_module('mod')
        module_with_imports = self.import_tools.module_imports(pymod)
        imports = module_with_imports.get_used_imports(pymod)
        self.assertEqual(1, len(imports))
        self.assertEqual('import pkg', imports[0].get_import_statement())

    def test_simple_getting_used_imports2(self):
        self.mod.write('import pkg\ndef a_func():\n    print(pkg)\n')
        pymod = self.project.get_module('mod')
        module_with_imports = self.import_tools.module_imports(pymod)
        imports = module_with_imports.get_used_imports(pymod)
        self.assertEqual(1, len(imports))
        self.assertEqual('import pkg', imports[0].get_import_statement())

    def test_getting_used_imports_for_nested_scopes(self):
        self.mod.write('import pkg1\nprint(pkg1)\n'
                       'def a_func():\n    pass\nprint(pkg1)\n')
        pymod = self.project.get_module('mod')
        module_with_imports = self.import_tools.module_imports(pymod)
        imports = module_with_imports.get_used_imports(
            pymod['a_func'].get_object())
        self.assertEqual(0, len(imports))

    def test_getting_used_imports_for_nested_scopes2(self):
        self.mod.write('from pkg1 import mod1\ndef a_func():'
                       '\n    print(mod1)\n')
        pymod = self.project.get_module('mod')
        module_with_imports = self.import_tools.module_imports(pymod)
        imports = module_with_imports.get_used_imports(
            pymod['a_func'].get_object())
        self.assertEqual(1, len(imports))
        self.assertEqual('from pkg1 import mod1',
                          imports[0].get_import_statement())

    def test_empty_removing_unused_imports(self):
        self.mod.write('import pkg1\nprint(pkg1)\n')
        pymod = self.project.get_module('mod')
        module_with_imports = self.import_tools.module_imports(pymod)
        module_with_imports.remove_unused_imports()
        self.assertEqual('import pkg1\nprint(pkg1)\n',
                          module_with_imports.get_changed_source())

    def test_simple_removing_unused_imports(self):
        self.mod.write('import pkg1\n\n')
        pymod = self.project.get_module('mod')
        module_with_imports = self.import_tools.module_imports(pymod)
        module_with_imports.remove_unused_imports()
        self.assertEqual('', module_with_imports.get_changed_source())

    def test_simple_removing_unused_imports_for_froms(self):
        self.mod1.write('def a_func():\n    pass'
                        '\ndef another_func():\n    pass\n')
        self.mod.write('from pkg1.mod1 import a_func, '
                       'another_func\n\na_func()\n')
        pymod = self.project.get_module('mod')
        module_with_imports = self.import_tools.module_imports(pymod)
        module_with_imports.remove_unused_imports()
        self.assertEqual('from pkg1.mod1 import a_func\n\na_func()\n',
                          module_with_imports.get_changed_source())

    def test_simple_removing_unused_imports_for_from_stars(self):
        self.mod.write('from pkg1.mod1 import *\n\n')
        pymod = self.project.get_module('mod')
        module_with_imports = self.import_tools.module_imports(pymod)
        module_with_imports.remove_unused_imports()
        self.assertEqual('', module_with_imports.get_changed_source())

    def test_simple_removing_unused_imports_for_nested_modules(self):
        self.mod1.write('def a_func():\n    pass\n')
        self.mod.write('import pkg1.mod1\npkg1.mod1.a_func()')
        pymod = self.project.get_module('mod')
        module_with_imports = self.import_tools.module_imports(pymod)
        module_with_imports.remove_unused_imports()
        self.assertEqual('import pkg1.mod1\npkg1.mod1.a_func()',
                          module_with_imports.get_changed_source())

    def test_removing_unused_imports_and_functions_of_the_same_name(self):
        self.mod.write('def a_func():\n    pass\ndef a_func():\n    pass\n')
        pymod = self.project.get_module('mod')
        module_with_imports = self.import_tools.module_imports(pymod)
        module_with_imports.remove_unused_imports()
        self.assertEqual('def a_func():\n    pass\ndef a_func():\n    pass\n',
                          module_with_imports.get_changed_source())

    def test_removing_unused_imports_for_from_import_with_as(self):
        self.mod.write('a_var = 1\n')
        self.mod1.write('from mod import a_var as myvar\na_var = myvar\n')
        pymod = self.project.get_pymodule(self.mod1)
        module_with_imports = self.import_tools.module_imports(pymod)
        module_with_imports.remove_unused_imports()
        self.assertEqual('from mod import a_var as myvar\na_var = myvar\n',
                          module_with_imports.get_changed_source())

    def test_not_removing_imports_that_conflict_with_class_names(self):
        code = 'import pkg1\nclass A(object):\n    pkg1 = 0\n' \
               '    def f(self):\n        a_var = pkg1\n'
        self.mod.write(code)
        pymod = self.project.get_module('mod')
        module_with_imports = self.import_tools.module_imports(pymod)
        module_with_imports.remove_unused_imports()
        self.assertEqual(code, module_with_imports.get_changed_source())

    def test_adding_imports(self):
        self.mod.write('\n')
        pymod = self.project.get_module('mod')
        module_with_imports = self.import_tools.module_imports(pymod)
        new_import = self.import_tools.get_import(self.mod1)
        module_with_imports.add_import(new_import)
        self.assertEqual('import pkg1.mod1\n',
                          module_with_imports.get_changed_source())

    def test_adding_imports_no_pull_to_top(self):
        self.mod.write('import pkg2.mod3\nclass A(object):\n    pass\n\n'
                       'import pkg2.mod2\n')
        pymod = self.project.get_module('mod')
        self.project.prefs['pull_imports_to_top'] = False
        module_with_imports = self.import_tools.module_imports(pymod)
        new_import = self.import_tools.get_import(self.mod1)
        module_with_imports.add_import(new_import)
        self.assertEqual('import pkg2.mod3\nclass A(object):\n    pass\n\n'
                          'import pkg2.mod2\nimport pkg1.mod1\n',
                          module_with_imports.get_changed_source())

    def test_adding_from_imports(self):
        self.mod1.write('def a_func():\n    pass\n'
                        'def another_func():\n    pass\n')
        self.mod.write('from pkg1.mod1 import a_func\n')
        pymod = self.project.get_module('mod')
        module_with_imports = self.import_tools.module_imports(pymod)
        new_import = self.import_tools.get_from_import(
            self.mod1, 'another_func')
        module_with_imports.add_import(new_import)
        self.assertEqual('from pkg1.mod1 import a_func, another_func\n',
                          module_with_imports.get_changed_source())

    def test_adding_to_star_imports(self):
        self.mod1.write('def a_func():\n    pass'
                        '\ndef another_func():\n    pass\n')
        self.mod.write('from pkg1.mod1 import *\n')
        pymod = self.project.get_module('mod')
        module_with_imports = self.import_tools.module_imports(pymod)
        new_import = self.import_tools.get_from_import(
            self.mod1, 'another_func')
        module_with_imports.add_import(new_import)
        self.assertEqual('from pkg1.mod1 import *\n',
                          module_with_imports.get_changed_source())

    def test_adding_star_imports(self):
        self.mod1.write('def a_func():\n    pass\n'
                        'def another_func():\n    pass\n')
        self.mod.write('from pkg1.mod1 import a_func\n')
        pymod = self.project.get_module('mod')
        module_with_imports = self.import_tools.module_imports(pymod)
        new_import = self.import_tools.get_from_import(self.mod1, '*')
        module_with_imports.add_import(new_import)
        self.assertEqual('from pkg1.mod1 import *\n',
                          module_with_imports.get_changed_source())

    def test_adding_imports_and_preserving_spaces_after_imports(self):
        self.mod.write('import pkg1\n\n\nprint(pkg1)\n')
        pymod = self.project.get_module('mod')
        module_with_imports = self.import_tools.module_imports(pymod)
        new_import = self.import_tools.get_import(self.pkg2)
        module_with_imports.add_import(new_import)
        self.assertEqual('import pkg1\nimport pkg2\n\n\nprint(pkg1)\n',
                          module_with_imports.get_changed_source())

    def test_not_changing_the_format_of_unchanged_imports(self):
        self.mod1.write('def a_func():\n    pass\n'
                        'def another_func():\n    pass\n')
        self.mod.write('from pkg1.mod1 import (a_func,\n    another_func)\n')
        pymod = self.project.get_module('mod')
        module_with_imports = self.import_tools.module_imports(pymod)
        self.assertEqual(
            'from pkg1.mod1 import (a_func,\n    another_func)\n',
            module_with_imports.get_changed_source())

    def test_not_changing_the_format_of_unchanged_imports2(self):
        self.mod1.write('def a_func():\n    pass\n'
                        'def another_func():\n    pass\n')
        self.mod.write('from pkg1.mod1 import (a_func)\na_func()\n')
        pymod = self.project.get_module('mod')
        module_with_imports = self.import_tools.module_imports(pymod)
        module_with_imports.remove_unused_imports()
        self.assertEqual('from pkg1.mod1 import (a_func)\na_func()\n',
                          module_with_imports.get_changed_source())

    def test_removing_unused_imports_and_reoccuring_names(self):
        self.mod1.write('def a_func():\n    pass\n'
                        'def another_func():\n    pass\n')
        self.mod.write('from pkg1.mod1 import *\n'
                       'from pkg1.mod1 import a_func\na_func()\n')
        pymod = self.project.get_module('mod')
        module_with_imports = self.import_tools.module_imports(pymod)
        module_with_imports.remove_unused_imports()
        self.assertEqual('from pkg1.mod1 import *\na_func()\n',
                          module_with_imports.get_changed_source())

    def test_removing_unused_imports_and_reoccuring_names2(self):
        self.mod.write('import pkg2.mod2\nimport pkg2.mod3\n'
                       'print(pkg2.mod2, pkg2.mod3)')
        pymod = self.project.get_module('mod')
        module_with_imports = self.import_tools.module_imports(pymod)
        module_with_imports.remove_unused_imports()
        self.assertEqual(
            'import pkg2.mod2\nimport pkg2.mod3\nprint(pkg2.mod2, pkg2.mod3)',
            module_with_imports.get_changed_source())

    def test_removing_unused_imports_and_common_packages(self):
        self.mod.write('import pkg1.mod1\nimport pkg1'
                       '\nprint(pkg1, pkg1.mod1)\n')
        pymod = self.project.get_module('mod')
        module_with_imports = self.import_tools.module_imports(pymod)
        module_with_imports.remove_unused_imports()
        self.assertEqual('import pkg1.mod1\nprint(pkg1, pkg1.mod1)\n',
                          module_with_imports.get_changed_source())

    def test_removing_unused_imports_and_common_packages_reversed(self):
        self.mod.write('import pkg1\nimport pkg1.mod1'
                       '\nprint(pkg1, pkg1.mod1)\n')
        pymod = self.project.get_module('mod')
        module_with_imports = self.import_tools.module_imports(pymod)
        module_with_imports.remove_duplicates()
        self.assertEqual('import pkg1.mod1\nprint(pkg1, pkg1.mod1)\n',
                          module_with_imports.get_changed_source())

    def test_removing_unused_imports_and_common_packages2(self):
        self.mod.write('import pkg1.mod1\nimport pkg1.mod2\nprint(pkg1)\n')
        pymod = self.project.get_module('mod')
        module_with_imports = self.import_tools.module_imports(pymod)
        module_with_imports.remove_unused_imports()
        self.assertEqual('import pkg1.mod1\nprint(pkg1)\n',
                          module_with_imports.get_changed_source())

    def test_removing_unused_imports_and_froms(self):
        self.mod1.write('def func1():\n    pass\n')
        self.mod.write('from pkg1.mod1 import func1\n')
        pymod = self.project.get_module('mod')
        module_with_imports = self.import_tools.module_imports(pymod)
        module_with_imports.remove_unused_imports()
        self.assertEqual('', module_with_imports.get_changed_source())

    def test_removing_unused_imports_and_froms2(self):
        self.mod1.write('def func1():\n    pass\n')
        self.mod.write('from pkg1.mod1 import func1\nfunc1()')
        pymod = self.project.get_module('mod')
        module_with_imports = self.import_tools.module_imports(pymod)
        module_with_imports.remove_unused_imports()
        self.assertEqual('from pkg1.mod1 import func1\nfunc1()',
                          module_with_imports.get_changed_source())

    def test_removing_unused_imports_and_froms3(self):
        self.mod1.write('def func1():\n    pass\n')
        self.mod.write('from pkg1.mod1 import func1\n'
                       'def a_func():\n    func1()\n')
        pymod = self.project.get_module('mod')
        module_with_imports = self.import_tools.module_imports(pymod)
        module_with_imports.remove_unused_imports()
        self.assertEqual(
            'from pkg1.mod1 import func1\ndef a_func():\n    func1()\n',
            module_with_imports.get_changed_source())

    def test_removing_unused_imports_and_froms4(self):
        self.mod1.write('def func1():\n    pass\n')
        self.mod.write('from pkg1.mod1 import func1\nclass A(object):\n'
                       '    def a_func(self):\n        func1()\n')
        pymod = self.project.get_module('mod')
        module_with_imports = self.import_tools.module_imports(pymod)
        module_with_imports.remove_unused_imports()
        self.assertEqual('from pkg1.mod1 import func1\nclass A(object):\n'
                          '    def a_func(self):\n        func1()\n',
                          module_with_imports.get_changed_source())

    def test_removing_unused_imports_and_getting_attributes(self):
        self.mod1.write('class A(object):\n    def f(self):\n        pass\n')
        self.mod.write('from pkg1.mod1 import A\nvar = A().f()')
        pymod = self.project.get_module('mod')
        module_with_imports = self.import_tools.module_imports(pymod)
        module_with_imports.remove_unused_imports()
        self.assertEqual('from pkg1.mod1 import A\nvar = A().f()',
                          module_with_imports.get_changed_source())

    def test_removing_unused_imports_function_parameters(self):
        self.mod1.write('def func1():\n    pass\n')
        self.mod.write('import pkg1\ndef a_func(pkg1):\n    my_var = pkg1\n')
        pymod = self.project.get_module('mod')
        module_with_imports = self.import_tools.module_imports(pymod)
        module_with_imports.remove_unused_imports()
        self.assertEqual('def a_func(pkg1):\n    my_var = pkg1\n',
                          module_with_imports.get_changed_source())

    def test_trivial_expanding_star_imports(self):
        self.mod1.write('def a_func():\n    pass\n'
                        'def another_func():\n    pass\n')
        self.mod.write('from pkg1.mod1 import *\n')
        pymod = self.project.get_module('mod')
        module_with_imports = self.import_tools.module_imports(pymod)
        module_with_imports.expand_stars()
        self.assertEqual('', module_with_imports.get_changed_source())

    def test_expanding_star_imports(self):
        self.mod1.write('def a_func():\n    pass\n'
                        'def another_func():\n    pass\n')
        self.mod.write('from pkg1.mod1 import *\na_func()\n')
        pymod = self.project.get_module('mod')
        module_with_imports = self.import_tools.module_imports(pymod)
        module_with_imports.expand_stars()
        self.assertEqual('from pkg1.mod1 import a_func\na_func()\n',
                          module_with_imports.get_changed_source())

    def test_removing_duplicate_imports(self):
        self.mod.write('import pkg1\nimport pkg1\n')
        pymod = self.project.get_module('mod')
        module_with_imports = self.import_tools.module_imports(pymod)
        module_with_imports.remove_duplicates()
        self.assertEqual('import pkg1\n',
                          module_with_imports.get_changed_source())

    def test_removing_duplicates_and_reoccuring_names(self):
        self.mod.write('import pkg2.mod2\nimport pkg2.mod3\n')
        pymod = self.project.get_module('mod')
        module_with_imports = self.import_tools.module_imports(pymod)
        module_with_imports.remove_duplicates()
        self.assertEqual('import pkg2.mod2\nimport pkg2.mod3\n',
                          module_with_imports.get_changed_source())

    def test_removing_duplicate_imports_for_froms(self):
        self.mod1.write(
            'def a_func():\n    pass\ndef another_func():\n    pass\n')
        self.mod.write('from pkg1 import a_func\n'
                       'from pkg1 import a_func, another_func\n')
        pymod = self.project.get_module('mod')
        module_with_imports = self.import_tools.module_imports(pymod)
        module_with_imports.remove_duplicates()
        self.assertEqual('from pkg1 import a_func, another_func\n',
                          module_with_imports.get_changed_source())

    def test_transforming_froms_to_normal_changing_imports(self):
        self.mod1.write('def a_func():\n    pass\n')
        self.mod.write('from pkg1.mod1 import a_func\nprint(a_func)\n')
        pymod = self.project.get_module('mod')
        changed_module = self.import_tools.froms_to_imports(pymod)
        self.assertEqual('import pkg1.mod1\nprint(pkg1.mod1.a_func)\n',
                          changed_module)

    def test_transforming_froms_to_normal_changing_occurances(self):
        self.mod1.write('def a_func():\n    pass\n')
        self.mod.write('from pkg1.mod1 import a_func\na_func()')
        pymod = self.project.get_module('mod')
        changed_module = self.import_tools.froms_to_imports(pymod)
        self.assertEqual('import pkg1.mod1\npkg1.mod1.a_func()',
                          changed_module)

    def test_transforming_froms_to_normal_for_multi_imports(self):
        self.mod1.write('def a_func():\n    pass\n'
                        'def another_func():\n    pass\n')
        self.mod.write('from pkg1.mod1 import *\na_func()\nanother_func()\n')
        pymod = self.project.get_module('mod')
        changed_module = self.import_tools.froms_to_imports(pymod)
        self.assertEqual(
            'import pkg1.mod1\npkg1.mod1.a_func()\npkg1.mod1.another_func()\n',
            changed_module)

    def test_transform_froms_to_norm_for_multi_imports_inside_parens(self):
        self.mod1.write('def a_func():\n    pass\n'
                        'def another_func():\n    pass\n')
        self.mod.write('from pkg1.mod1 import (a_func, \n    another_func)'
                       '\na_func()\nanother_func()\n')
        pymod = self.project.get_module('mod')
        changed_module = self.import_tools.froms_to_imports(pymod)
        self.assertEqual(
            'import pkg1.mod1\npkg1.mod1.a_func()\npkg1.mod1.another_func()\n',
            changed_module)

    def test_transforming_froms_to_normal_from_stars(self):
        self.mod1.write('def a_func():\n    pass\n')
        self.mod.write('from pkg1.mod1 import *\na_func()\n')
        pymod = self.project.get_module('mod')
        changed_module = self.import_tools.froms_to_imports(pymod)
        self.assertEqual('import pkg1.mod1\npkg1.mod1.a_func()\n',
                          changed_module)

    def test_transforming_froms_to_normal_from_stars2(self):
        self.mod1.write('a_var = 10')
        self.mod.write('import pkg1.mod1\nfrom pkg1.mod1 import a_var\n'
                       'def a_func():\n    print(pkg1.mod1, a_var)\n')
        pymod = self.project.get_module('mod')
        changed_module = self.import_tools.froms_to_imports(pymod)
        self.assertEqual('import pkg1.mod1\n'
                          'def a_func():\n    '
                          'print(pkg1.mod1, pkg1.mod1.a_var)\n',
                          changed_module)

    def test_transforming_froms_to_normal_from_with_alias(self):
        self.mod1.write('def a_func():\n    pass\n')
        self.mod.write(
            'from pkg1.mod1 import a_func as another_func\nanother_func()\n')
        pymod = self.project.get_module('mod')
        changed_module = self.import_tools.froms_to_imports(pymod)
        self.assertEqual('import pkg1.mod1\npkg1.mod1.a_func()\n',
                          changed_module)

    def test_transforming_froms_to_normal_for_relatives(self):
        self.mod2.write('def a_func():\n    pass\n')
        self.mod3.write('from mod2 import *\na_func()\n')
        pymod = self.project.get_pymodule(self.mod3)
        changed_module = self.import_tools.froms_to_imports(pymod)
        self.assertEqual('import pkg2.mod2\npkg2.mod2.a_func()\n',
                          changed_module)

    def test_transforming_froms_to_normal_for_os_path(self):
        self.mod.write('from os import path\npath.exists(\'.\')\n')
        pymod = self.project.get_pymodule(self.mod)
        changed_module = self.import_tools.froms_to_imports(pymod)
        self.assertEqual('import os\nos.path.exists(\'.\')\n', changed_module)

    def test_transform_relatives_imports_to_abs_imports_doing_nothing(self):
        self.mod2.write('from pkg1 import mod1\nimport mod1\n')
        pymod = self.project.get_pymodule(self.mod2)
        self.assertEqual('from pkg1 import mod1\nimport mod1\n',
                          self.import_tools.relatives_to_absolutes(pymod))

    def test_transform_relatives_to_absolute_imports_for_normal_imports(self):
        self.mod2.write('import mod3\n')
        pymod = self.project.get_pymodule(self.mod2)
        self.assertEqual('import pkg2.mod3\n',
                          self.import_tools.relatives_to_absolutes(pymod))

    def test_transform_relatives_imports_to_absolute_imports_for_froms(self):
        self.mod3.write('def a_func():\n    pass\n')
        self.mod2.write('from mod3 import a_func\n')
        pymod = self.project.get_pymodule(self.mod2)
        self.assertEqual('from pkg2.mod3 import a_func\n',
                          self.import_tools.relatives_to_absolutes(pymod))

    @testutils.only_for('2.5')
    def test_transform_rel_imports_to_abs_imports_for_new_relatives(self):
        self.mod3.write('def a_func():\n    pass\n')
        self.mod2.write('from .mod3 import a_func\n')
        pymod = self.project.get_pymodule(self.mod2)
        self.assertEqual('from pkg2.mod3 import a_func\n',
                          self.import_tools.relatives_to_absolutes(pymod))

    def test_transform_relatives_to_absolute_imports_for_normal_imports2(self):
        self.mod2.write('import mod3\nprint(mod3)')
        pymod = self.project.get_pymodule(self.mod2)
        self.assertEqual('import pkg2.mod3\nprint(pkg2.mod3)',
                          self.import_tools.relatives_to_absolutes(pymod))

    def test_transform_relatives_to_absolute_imports_for_aliases(self):
        self.mod2.write('import mod3 as mod3\nprint(mod3)')
        pymod = self.project.get_pymodule(self.mod2)
        self.assertEqual('import pkg2.mod3 as mod3\nprint(mod3)',
                          self.import_tools.relatives_to_absolutes(pymod))

    def test_organizing_imports(self):
        self.mod1.write('import mod1\n')
        pymod = self.project.get_pymodule(self.mod1)
        self.assertEqual('', self.import_tools.organize_imports(pymod))

    def test_organizing_imports_without_deduplication(self):
        contents = 'from pkg2 import mod2\nfrom pkg2 import mod3\n'
        self.mod.write(contents)
        pymod = self.project.get_pymodule(self.mod)
        self.project.prefs['split_imports'] = True
        self.assertEqual(contents,
                          self.import_tools.organize_imports(pymod,
                                                             unused=False))

    def test_splitting_imports(self):
        self.mod.write('from pkg1 import mod1\nfrom pkg2 import mod2, mod3\n')
        pymod = self.project.get_pymodule(self.mod)
        self.project.prefs['split_imports'] = True
        self.assertEqual('from pkg1 import mod1\nfrom pkg2 import mod2\n'
                          'from pkg2 import mod3\n',
                          self.import_tools.organize_imports(pymod,
                                                             unused=False))

    def test_splitting_imports_no_pull_to_top(self):
        self.mod.write('from pkg2 import mod3, mod4\n'
                       'from pkg1 import mod2\nfrom pkg1 import mod1\n')
        pymod = self.project.get_pymodule(self.mod)
        self.project.prefs['split_imports'] = True
        self.project.prefs['pull_imports_to_top'] = False
        self.assertEqual('from pkg1 import mod2\nfrom pkg1 import mod1\n'
                          'from pkg2 import mod3\nfrom pkg2 import mod4\n',
                          self.import_tools.organize_imports(pymod,
                                                             sort=False,
                                                             unused=False))

    def test_splitting_imports_with_filter(self):
        self.mod.write('from pkg1 import mod1, mod2\n'
                       'from pkg2 import mod3, mod4\n')
        pymod = self.project.get_pymodule(self.mod)
        self.project.prefs['split_imports'] = True

        def import_filter(stmt):
          return stmt.import_info.module_name == 'pkg1'

        self.assertEqual(
            'from pkg1 import mod1\nfrom pkg1 import mod2\n'
            'from pkg2 import mod3, mod4\n',
            self.import_tools.organize_imports(pymod, unused=False,
                                               import_filter=import_filter))

    def test_splitting_duplicate_imports(self):
        self.mod.write('from pkg2 import mod1\nfrom pkg2 import mod1, mod2\n')
        pymod = self.project.get_pymodule(self.mod)
        self.project.prefs['split_imports'] = True
        self.assertEqual('from pkg2 import mod1\nfrom pkg2 import mod2\n',
                          self.import_tools.organize_imports(pymod,
                                                             unused=False))

    def test_splitting_duplicate_imports2(self):
        self.mod.write('from pkg2 import mod1, mod3\n'
                       'from pkg2 import mod1, mod2\n'
                       'from pkg2 import mod2, mod3\n')
        pymod = self.project.get_pymodule(self.mod)
        self.project.prefs['split_imports'] = True
        self.assertEqual('from pkg2 import mod1\nfrom pkg2 import mod2\n'
                          'from pkg2 import mod3\n',
                          self.import_tools.organize_imports(pymod,
                                                             unused=False))

    def test_removing_self_imports(self):
        self.mod.write('import mod\nmod.a_var = 1\nprint(mod.a_var)\n')
        pymod = self.project.get_pymodule(self.mod)
        self.assertEqual('a_var = 1\nprint(a_var)\n',
                          self.import_tools.organize_imports(pymod))

    def test_removing_self_imports2(self):
        self.mod1.write('import pkg1.mod1\npkg1.mod1.a_var = 1\n'
                        'print(pkg1.mod1.a_var)\n')
        pymod = self.project.get_pymodule(self.mod1)
        self.assertEqual('a_var = 1\nprint(a_var)\n',
                          self.import_tools.organize_imports(pymod))

    def test_removing_self_imports_with_as(self):
        self.mod.write('import mod as mymod\n'
                       'mymod.a_var = 1\nprint(mymod.a_var)\n')
        pymod = self.project.get_pymodule(self.mod)
        self.assertEqual('a_var = 1\nprint(a_var)\n',
                          self.import_tools.organize_imports(pymod))

    def test_removing_self_imports_for_froms(self):
        self.mod1.write('from pkg1 import mod1\n'
                        'mod1.a_var = 1\nprint(mod1.a_var)\n')
        pymod = self.project.get_pymodule(self.mod1)
        self.assertEqual('a_var = 1\nprint(a_var)\n',
                          self.import_tools.organize_imports(pymod))

    def test_removing_self_imports_for_froms_with_as(self):
        self.mod1.write('from pkg1 import mod1 as mymod\n'
                        'mymod.a_var = 1\nprint(mymod.a_var)\n')
        pymod = self.project.get_pymodule(self.mod1)
        self.assertEqual('a_var = 1\nprint(a_var)\n',
                          self.import_tools.organize_imports(pymod))

    def test_removing_self_imports_for_froms2(self):
        self.mod.write('from mod import a_var\na_var = 1\nprint(a_var)\n')
        pymod = self.project.get_pymodule(self.mod)
        self.assertEqual('a_var = 1\nprint(a_var)\n',
                          self.import_tools.organize_imports(pymod))

    def test_removing_self_imports_for_froms3(self):
        self.mod.write('from mod import a_var\na_var = 1\nprint(a_var)\n')
        pymod = self.project.get_pymodule(self.mod)
        self.assertEqual('a_var = 1\nprint(a_var)\n',
                          self.import_tools.organize_imports(pymod))

    def test_removing_self_imports_for_froms4(self):
        self.mod.write('from mod import a_var as myvar\n'
                       'a_var = 1\nprint(myvar)\n')
        pymod = self.project.get_pymodule(self.mod)
        self.assertEqual('a_var = 1\nprint(a_var)\n',
                          self.import_tools.organize_imports(pymod))

    def test_removing_self_imports_with_no_dot_after_mod(self):
        self.mod.write('import mod\nprint(mod)\n')
        pymod = self.project.get_pymodule(self.mod)
        self.assertEqual('import mod\n\n\nprint(mod)\n',
                          self.import_tools.organize_imports(pymod))

    def test_removing_self_imports_with_no_dot_after_mod2(self):
        self.mod.write('import mod\na_var = 1\n'
                       'print(mod\\\n     \\\n     .var)\n\n')
        pymod = self.project.get_pymodule(self.mod)
        self.assertEqual('a_var = 1\nprint(var)\n\n',
                          self.import_tools.organize_imports(pymod))

    def test_removing_self_imports_for_from_import_star(self):
        self.mod.write('from mod import *\na_var = 1\nprint(myvar)\n')
        pymod = self.project.get_pymodule(self.mod)
        self.assertEqual('a_var = 1\nprint(myvar)\n',
                          self.import_tools.organize_imports(pymod))

    def test_not_removing_future_imports(self):
        self.mod.write('from __future__ import division\n')
        pymod = self.project.get_pymodule(self.mod)
        self.assertEqual('from __future__ import division\n',
                          self.import_tools.organize_imports(pymod))

    def test_sorting_empty_imports(self):
        self.mod.write('')
        pymod = self.project.get_pymodule(self.mod)
        self.assertEqual('', self.import_tools.sort_imports(pymod))

    def test_sorting_one_import(self):
        self.mod.write('import pkg1.mod1\n')
        pymod = self.project.get_pymodule(self.mod)
        self.assertEqual('import pkg1.mod1\n',
                          self.import_tools.sort_imports(pymod))

    def test_sorting_imports_alphabetically(self):
        self.mod.write('import pkg2.mod2\nimport pkg1.mod1\n')
        pymod = self.project.get_pymodule(self.mod)
        self.assertEqual('import pkg1.mod1\nimport pkg2.mod2\n',
                          self.import_tools.sort_imports(pymod))

    def test_sorting_imports_purely_alphabetically(self):
        self.mod.write('from pkg2 import mod3 as mod0\n'
                       'import pkg2.mod2\nimport pkg1.mod1\n')
        pymod = self.project.get_pymodule(self.mod)
        self.project.prefs['sort_imports_alphabetically'] = True
        self.assertEqual('import pkg1.mod1\nimport pkg2.mod2\n'
                          'from pkg2 import mod3 as mod0\n',
                          self.import_tools.sort_imports(pymod))

    def test_sorting_imports_and_froms(self):
        self.mod.write('import pkg2.mod2\nfrom pkg1 import mod1\n')
        pymod = self.project.get_pymodule(self.mod)
        self.assertEqual('import pkg2.mod2\nfrom pkg1 import mod1\n',
                          self.import_tools.sort_imports(pymod))

    def test_sorting_imports_and_standard_modules(self):
        self.mod.write('import pkg1\nimport sys\n')
        pymod = self.project.get_pymodule(self.mod)
        self.assertEqual('import sys\n\nimport pkg1\n',
                          self.import_tools.sort_imports(pymod))

    def test_sorting_imports_and_standard_modules2(self):
        self.mod.write('import sys\n\nimport time\n')
        pymod = self.project.get_pymodule(self.mod)
        self.assertEqual('import sys\nimport time\n',
                          self.import_tools.sort_imports(pymod))

    def test_sorting_only_standard_modules(self):
        self.mod.write('import sys\n')
        pymod = self.project.get_pymodule(self.mod)
        self.assertEqual('import sys\n',
                          self.import_tools.sort_imports(pymod))

    def test_sorting_third_party(self):
        self.mod.write('import pkg1\nimport a_third_party\n')
        pymod = self.project.get_pymodule(self.mod)
        self.assertEqual('import a_third_party\n\nimport pkg1\n',
                          self.import_tools.sort_imports(pymod))

    def test_sorting_only_third_parties(self):
        self.mod.write('import a_third_party\na_var = 1\n')
        pymod = self.project.get_pymodule(self.mod)
        self.assertEqual('import a_third_party\n\n\na_var = 1\n',
                          self.import_tools.sort_imports(pymod))

    def test_simple_handling_long_imports(self):
        self.mod.write('import pkg1.mod1\n\n\nm = pkg1.mod1\n')
        pymod = self.project.get_pymodule(self.mod)
        self.assertEqual(
            'import pkg1.mod1\n\n\nm = pkg1.mod1\n',
            self.import_tools.handle_long_imports(pymod, maxdots=2))

    def test_handling_long_imports_for_many_dots(self):
        self.mod.write('import p1.p2.p3.m1\n\n\nm = p1.p2.p3.m1\n')
        pymod = self.project.get_pymodule(self.mod)
        self.assertEqual(
            'from p1.p2.p3 import m1\n\n\nm = m1\n',
            self.import_tools.handle_long_imports(pymod, maxdots=2))

    def test_handling_long_imports_for_their_length(self):
        self.mod.write('import p1.p2.p3.m1\n\n\nm = p1.p2.p3.m1\n')
        pymod = self.project.get_pymodule(self.mod)
        self.assertEqual(
            'import p1.p2.p3.m1\n\n\nm = p1.p2.p3.m1\n',
            self.import_tools.handle_long_imports(pymod, maxdots=3,
                                                  maxlength=20))

    def test_handling_long_imports_for_many_dots2(self):
        self.mod.write('import p1.p2.p3.m1\n\n\nm = p1.p2.p3.m1\n')
        pymod = self.project.get_pymodule(self.mod)
        self.assertEqual(
            'from p1.p2.p3 import m1\n\n\nm = m1\n',
            self.import_tools.handle_long_imports(pymod, maxdots=3,
                                                  maxlength=10))

    def test_handling_long_imports_with_one_letter_last(self):
        self.mod.write('import p1.p2.p3.l\n\n\nm = p1.p2.p3.l\n')
        pymod = self.project.get_pymodule(self.mod)
        self.assertEqual(
            'from p1.p2.p3 import l\n\n\nm = l\n',
            self.import_tools.handle_long_imports(pymod, maxdots=2))

    def test_empty_removing_unused_imports_and_eating_blank_lines(self):
        self.mod.write('import pkg1\nimport pkg2\n\n\nprint(pkg1)\n')
        pymod = self.project.get_module('mod')
        module_with_imports = self.import_tools.module_imports(pymod)
        module_with_imports.remove_unused_imports()
        self.assertEqual('import pkg1\n\n\nprint(pkg1)\n',
                          module_with_imports.get_changed_source())

    def test_sorting_imports_moving_to_top(self):
        self.mod.write('import mod\ndef f():\n    print(mod, pkg1, pkg2)\n'
                       'import pkg1\nimport pkg2\n')
        pymod = self.project.get_module('mod')
        self.assertEqual('import mod\nimport pkg1\nimport pkg2\n\n\n'
                          'def f():\n    print(mod, pkg1, pkg2)\n',
                          self.import_tools.sort_imports(pymod))

    def test_sorting_imports_moving_to_top2(self):
        self.mod.write('def f():\n    print(mod)\nimport mod\n')
        pymod = self.project.get_module('mod')
        self.assertEqual('import mod\n\n\ndef f():\n    print(mod)\n',
                          self.import_tools.sort_imports(pymod))

    # Sort pulls imports to the top anyway
    def test_sorting_imports_no_pull_to_top(self):
        code = ('import pkg2\ndef f():\n    print(mod, pkg1, pkg2)\n'
                'import pkg1\nimport mod\n')
        self.mod.write(code)
        pymod = self.project.get_module('mod')
        self.project.prefs['pull_imports_to_top'] = False
        self.assertEqual(
            'import mod\nimport pkg1\nimport pkg2\n\n\n'
            'def f():\n    print(mod, pkg1, pkg2)\n',
            self.import_tools.sort_imports(pymod))

    def test_sorting_imports_moving_to_top_and_module_docs(self):
        self.mod.write('"""\ndocs\n"""\ndef f():'
                       '\n    print(mod)\nimport mod\n')
        pymod = self.project.get_module('mod')
        self.assertEqual(
            '"""\ndocs\n"""\nimport mod\n\n\ndef f():\n    print(mod)\n',
            self.import_tools.sort_imports(pymod))

    def test_sorting_imports_moving_to_top_and_module_docs2(self):
        self.mod.write('"""\ndocs\n"""\n\n\nimport bbb\nimport aaa\n'
                       'def f():\n    print(mod)\nimport mod\n')
        pymod = self.project.get_module('mod')
        self.assertEqual(
            '"""\ndocs\n"""\n\n\nimport aaa\nimport bbb\n\n'
            'import mod\n\n\ndef f():\n    print(mod)\n',
            self.import_tools.sort_imports(pymod))

    def test_get_changed_source_preserves_blank_lines(self):
        self.mod.write(
            '__author__ = "author"\n\nimport aaa\n\nimport bbb\n\n'
            'def f():\n    print(mod)\n')
        pymod = self.project.get_module('mod')
        module_with_imports = self.import_tools.module_imports(pymod)
        self.assertEqual(
            'import aaa\n\nimport bbb\n\n__author__ = "author"\n\n'
            'def f():\n    print(mod)\n',
            module_with_imports.get_changed_source())

    def test_sorting_future_imports(self):
        self.mod.write('import os\nfrom __future__ import devision\n')
        pymod = self.project.get_module('mod')
        self.assertEqual(
            'from __future__ import devision\n\nimport os\n',
            self.import_tools.sort_imports(pymod))

    def test_organizing_imports_all_star(self):
        code = expected = dedent('''\
            from package import some_name


            __all__ = ["some_name"]
        ''')
        self.mod.write(code)
        pymod = self.project.get_pymodule(self.mod)
        self.assertEqual(
            expected,
            self.import_tools.organize_imports(pymod))

    def test_organizing_imports_all_star_with_variables(self):
        code = expected = dedent('''\
            from package import name_one, name_two


            if something():
                foo = 'name_one'
            else:
                foo = 'name_two'
            __all__ = [foo]
        ''')
        self.mod.write(code)
        pymod = self.project.get_pymodule(self.mod)
        self.assertEqual(
            expected,
            self.import_tools.organize_imports(pymod))

    def test_organizing_imports_all_star_with_inline_if(self):
        code = expected = dedent('''\
            from package import name_one, name_two


            __all__ = ['name_one' if something() else 'name_two']
        ''')
        self.mod.write(code)
        pymod = self.project.get_pymodule(self.mod)
        self.assertEqual(
            expected,
            self.import_tools.organize_imports(pymod))

    @testutils.only_for_versions_higher('3')
    def test_organizing_imports_all_star_tolerates_non_list_of_str_1(self):
        code = expected = dedent('''\
            from package import name_one, name_two


            foo = 'name_two'
            __all__ = [bar, *abc] + mylist
            __all__ = [foo, 'name_one', *abc]
            __all__ = [it for it in mylist]
        ''')
        self.mod.write(code)
        pymod = self.project.get_pymodule(self.mod)
        self.assertEqual(
            expected,
            self.import_tools.organize_imports(pymod))

    def test_organizing_imports_all_star_tolerates_non_list_of_str_2(self):
        code = expected = dedent('''\
            from package import name_one, name_two


            foo = 'name_two'
            __all__ = [foo, 3, 'name_one']
            __all__ = [it for it in mylist]
        ''')
        self.mod.write(code)
        pymod = self.project.get_pymodule(self.mod)
        self.assertEqual(
            expected,
            self.import_tools.organize_imports(pymod))

    @testutils.time_limit(60)
    def test_organizing_imports_all_star_no_infinite_loop(self):
        code = expected = dedent('''\
            from package import name_one, name_two


            foo = bar
            bar = foo
            __all__ = [foo, 'name_one', 'name_two']
        ''')
        self.mod.write(code)
        pymod = self.project.get_pymodule(self.mod)
        self.assertEqual(
            expected,
            self.import_tools.organize_imports(pymod))

    def test_customized_import_organization(self):
        self.mod.write('import sys\nimport sys\n')
        pymod = self.project.get_pymodule(self.mod)
        self.assertEqual(
            'import sys\n',
            self.import_tools.organize_imports(pymod, unused=False))

    def test_customized_import_organization2(self):
        self.mod.write('import sys\n')
        pymod = self.project.get_pymodule(self.mod)
        self.assertEqual(
            'import sys\n',
            self.import_tools.organize_imports(pymod, unused=False))

    def test_customized_import_organization3(self):
        self.mod.write('import sys\nimport mod\n\n\nvar = 1\nprint(mod.var)\n')
        pymod = self.project.get_pymodule(self.mod)
        self.assertEqual(
            'import sys\n\n\nvar = 1\nprint(var)\n',
            self.import_tools.organize_imports(pymod, unused=False))

    def test_trivial_filtered_expand_stars(self):
        self.pkg1.get_child('__init__.py').write('var1 = 1\n')
        self.pkg2.get_child('__init__.py').write('var2 = 1\n')
        self.mod.write('from pkg1 import *\nfrom pkg2 import *\n\n'
                       'print(var1, var2)\n')
        pymod = self.project.get_pymodule(self.mod)
        self.assertEqual(
            'from pkg1 import *\nfrom pkg2 import *\n\nprint(var1, var2)\n',
            self.import_tools.expand_stars(pymod, lambda stmt: False))

    def _line_filter(self, lineno):
        def import_filter(import_stmt):
            return import_stmt.start_line <= lineno < import_stmt.end_line
        return import_filter

    def test_filtered_expand_stars(self):
        self.pkg1.get_child('__init__.py').write('var1 = 1\n')
        self.pkg2.get_child('__init__.py').write('var2 = 1\n')
        self.mod.write('from pkg1 import *\nfrom pkg2 import *\n\n'
                       'print(var1, var2)\n')
        pymod = self.project.get_pymodule(self.mod)
        self.assertEqual(
            'from pkg1 import *\nfrom pkg2 import var2\n\nprint(var1, var2)\n',
            self.import_tools.expand_stars(pymod, self._line_filter(2)))

    def test_filtered_relative_to_absolute(self):
        self.mod3.write('var = 1')
        self.mod2.write('import mod3\n\nprint(mod3.var)\n')
        pymod = self.project.get_pymodule(self.mod2)
        self.assertEqual(
            'import mod3\n\nprint(mod3.var)\n',
            self.import_tools.relatives_to_absolutes(
                pymod, lambda stmt: False))
        self.assertEqual(
            'import pkg2.mod3\n\nprint(pkg2.mod3.var)\n',
            self.import_tools.relatives_to_absolutes(
                pymod, self._line_filter(1)))

    def test_filtered_froms_to_normals(self):
        self.pkg1.get_child('__init__.py').write('var1 = 1\n')
        self.pkg2.get_child('__init__.py').write('var2 = 1\n')
        self.mod.write('from pkg1 import var1\nfrom pkg2 import var2\n\n'
                       'print(var1, var2)\n')
        pymod = self.project.get_pymodule(self.mod)
        self.assertEqual(
            'from pkg1 import var1\nfrom pkg2 '
            'import var2\n\nprint(var1, var2)\n',
            self.import_tools.expand_stars(pymod, lambda stmt: False))
        self.assertEqual(
            'from pkg1 import var1\nimport pkg2\n\nprint(var1, pkg2.var2)\n',
            self.import_tools.froms_to_imports(pymod, self._line_filter(2)))

    def test_filtered_froms_to_normals2(self):
        self.pkg1.get_child('__init__.py').write('var1 = 1\n')
        self.pkg2.get_child('__init__.py').write('var2 = 1\n')
        self.mod.write('from pkg1 import *\nfrom pkg2 import *\n\n'
                       'print(var1, var2)\n')
        pymod = self.project.get_pymodule(self.mod)
        self.assertEqual(
            'from pkg1 import *\nimport pkg2\n\nprint(var1, pkg2.var2)\n',
            self.import_tools.froms_to_imports(pymod, self._line_filter(2)))

    def test_filtered_handle_long_imports(self):
        self.mod.write('import p1.p2.p3.m1\nimport pkg1.mod1\n\n\n'
                       'm = p1.p2.p3.m1, pkg1.mod1\n')
        pymod = self.project.get_pymodule(self.mod)
        self.assertEqual(
            'import p1.p2.p3.m1\nfrom pkg1 import mod1\n\n\n'
            'm = p1.p2.p3.m1, mod1\n',
            self.import_tools.handle_long_imports(
                pymod, maxlength=5,
                import_filter=self._line_filter(2)))

    def test_filtering_and_import_actions_with_more_than_one_phase(self):
        self.pkg1.get_child('__init__.py').write('var1 = 1\n')
        self.pkg2.get_child('__init__.py').write('var2 = 1\n')
        self.mod.write('from pkg1 import *\nfrom pkg2 import *\n\n'
                       'print(var2)\n')
        pymod = self.project.get_pymodule(self.mod)
        self.assertEqual(
            'from pkg2 import *\n\nprint(var2)\n',
            self.import_tools.expand_stars(pymod, self._line_filter(1)))

    def test_non_existent_module_and_used_imports(self):
        self.mod.write(
            'from does_not_exist import func\n\nfunc()\n')
        pymod = self.project.get_module('mod')

        module_with_imports = self.import_tools.module_imports(pymod)
        imports = module_with_imports.get_used_imports(pymod)
        self.assertEqual(1, len(imports))


class AddImportTest(unittest.TestCase):

    def setUp(self):
        super(AddImportTest, self).setUp()
        self.project = testutils.sample_project()

        self.mod1 = testutils.create_module(self.project, 'mod1')
        self.mod2 = testutils.create_module(self.project, 'mod2')
        self.pkg = testutils.create_package(self.project, 'pkg')
        self.mod3 = testutils.create_module(self.project, 'mod3', self.pkg)

    def tearDown(self):
        testutils.remove_project(self.project)
        super(AddImportTest, self).tearDown()

    def test_normal_imports(self):
        self.mod2.write('myvar = None\n')
        self.mod1.write('\n')
        pymod = self.project.get_module('mod1')
        result, name = add_import(self.project, pymod, 'mod2', 'myvar')
        self.assertEqual('import mod2\n', result)
        self.assertEqual('mod2.myvar', name)

    def test_not_reimporting_a_name(self):
        self.mod2.write('myvar = None\n')
        self.mod1.write('from mod2 import myvar\n')
        pymod = self.project.get_module('mod1')
        result, name = add_import(self.project, pymod, 'mod2', 'myvar')
        self.assertEqual('from mod2 import myvar\n', result)
        self.assertEqual('myvar', name)

    def test_adding_import_when_siblings_are_imported(self):
        self.mod2.write('var1 = None\nvar2 = None\n')
        self.mod1.write('from mod2 import var1\n')
        pymod = self.project.get_module('mod1')
        result, name = add_import(self.project, pymod, 'mod2', 'var2')
        self.assertEqual('from mod2 import var1, var2\n', result)
        self.assertEqual('var2', name)

    def test_adding_import_when_the_package_is_imported(self):
        self.pkg.get_child('__init__.py').write('var1 = None\n')
        self.mod3.write('var2 = None\n')
        self.mod1.write('from pkg import var1\n')
        pymod = self.project.get_module('mod1')
        result, name = add_import(self.project, pymod, 'pkg.mod3', 'var2')
        self.assertEqual('from pkg import var1, mod3\n', result)
        self.assertEqual('mod3.var2', name)

    def test_adding_import_for_modules_instead_of_names(self):
        self.pkg.get_child('__init__.py').write('var1 = None\n')
        self.mod3.write('\n')
        self.mod1.write('from pkg import var1\n')
        pymod = self.project.get_module('mod1')
        result, name = add_import(self.project, pymod, 'pkg.mod3', None)
        self.assertEqual('from pkg import var1, mod3\n', result)
        self.assertEqual('mod3', name)

    def test_adding_import_for_modules_with_normal_duplicate_imports(self):
        self.pkg.get_child('__init__.py').write('var1 = None\n')
        self.mod3.write('\n')
        self.mod1.write('import pkg.mod3\n')
        pymod = self.project.get_module('mod1')
        result, name = add_import(self.project, pymod, 'pkg.mod3', None)
        self.assertEqual('import pkg.mod3\n', result)
        self.assertEqual('pkg.mod3', name)


def suite():
    result = unittest.TestSuite()
    result.addTests(unittest.makeSuite(ImportUtilsTest))
    result.addTests(unittest.makeSuite(AddImportTest))
    return result

if __name__ == '__main__':
    unittest.main()



import sys
import ast
# from rope.base import ast

PY2 = sys.version_info[0] == 2
PY27 = sys.version_info[0:2] >= (2, 7)
PY3 = sys.version_info[0] == 3
PY34 = sys.version_info[0:2] >= (3, 4)
PY36 = sys.version_info[0:2] >= (3, 6)

try:
    str = unicode
except NameError:  # PY3

    str = str
    string_types = (str,)
    import builtins
    ast_arg_type = ast.arg

    def execfile(fn, global_vars=None, local_vars=None):
        with open(fn) as f:
            code = compile(f.read(), fn, 'exec')
            exec(code, global_vars or {}, local_vars)

    def get_ast_arg_arg(node):
        if isinstance(node, string_types):  # TODO: G21: Understand the Algorithm (Where it's used?)
            return node
        return node.arg

    def get_ast_with_items(node):
        return node.items

else:  # PY2

    string_types = (basestring,)
    builtins = __import__('__builtin__')
    ast_arg_type = ast.Name
    execfile = execfile

    def get_ast_arg_arg(node):
        if isinstance(node, string_types):  # Python2 arguments.vararg, arguments.kwarg
            return node
        return node.id

    def get_ast_with_items(node):
        return [node]
